#!/usr/bin/python

import sqlite3

import dataextract
import mptxn_plot
import stupidplot

LOCK = "data/conflict-lock.csv"


# This *was* supposed to be generic. Oops.
def extractConflicts(path, name):
    # Read in the table
    table = dataextract.readCSVTable(path)

    # Expand each repeated data point
    limit = len(table[0])
    table = dataextract.expandRows(table, [0, 1], range(2, limit))
    table.insert(0, ["conflict_percent", "multipartition_percent", "txns_per_sec"])

    connection = sqlite3.connect(":memory:")
    c = connection.cursor()

    # Create the table
    c.execute("create table results (%s);" % (", ".join(table[0])))

    # Import the data
    insert_template = "?," * len(table[0])
    insert_template = "insert into results values (%s)" % (insert_template[:-1])
    #~ print insert_template
    for row in table[1:]:
        c.execute(insert_template, row)

    datasets = []
    for v in list(c.execute("select distinct conflict_percent from results")):
        conflict_percent = v[0]
        #~ legends.append("Lock %.0f%% conflict" % (conflict_percent))
        table = [[mptxn_plot.X_AXIS, "%s %.0f%% conflict" % (name, conflict_percent), "- conf", "+ conf"]]
        for y in list(c.execute("select distinct multipartition_percent from results where "
                "conflict_percent = ? order by multipartition_percent asc", [conflict_percent])):
            y = y[0]
            x_values = []
            for x in c.execute("select txns_per_sec from results where conflict_percent = ? and multipartition_percent = ?", (conflict_percent, y)):
                x_values.append(x[0])

            stats = stupidplot.stats(x_values)
            table.append([y, stats[0], stats[0]-stats[-1], stats[0]+stats[-1]])
            #~ print table[-1]
        datasets.append(table)
    
    return datasets


if __name__ == "__main__":
    output_path = mptxn_plot.get_output()

    datasets = extractConflicts(LOCK, "Locking")

    # HACK: Remove the 40% and 80% lines
    REMOVE = (40, 80)
    filtered = []
    for i, dataset in enumerate(datasets):
    
        found = False
        for v in REMOVE:
            if str(v) in dataset[0][1]:
                found = True
                break
        if not found:
            filtered.append(mptxn_plot.removeErrorBars(dataset))
    datasets = filtered

    for path, line_title in (mptxn_plot.INPUTS[0], mptxn_plot.INPUTS[1]):
        average, scatter = dataextract.extractTables(path, mptxn_plot.X_AXIS, mptxn_plot.Y_AXIS, line_title)
        datasets.append(mptxn_plot.removeErrorBars(average))

    options = dict(mptxn_plot.DEFAULT_PLOT_OPTIONS)
    for i in range(1, 5):
        options["colors"][i] = mptxn_plot.BLACK
    options["colors"][5] = mptxn_plot.BLUE
    options["colors"][6] = mptxn_plot.RED
    options["colors"][7] = mptxn_plot.GREEN
    stupidplot.gnuplotTable(datasets, output_path, options)
