#ifndef STLUTIL_H__
#define STLUTIL_H__

#include <algorithm>

// Deletes all elements in STL container.
template <typename T>
static void STLDeleteElements(T* container) {
    const typename T::iterator end = container->end();
    for (typename T::iterator i = container->begin(); i != end; ++i) {
        delete *i;
    }
    container->clear();
};

// Deletes all values (iterator->second) in STL container.
template <typename T>
static void STLDeleteValues(T* container) {
    const typename T::iterator end = container->end();
    for (typename T::iterator i = container->begin(); i != end; ++i) {
        delete i->second;
    }
    container->clear();
};

// Adds all the values in source to the end of destination using push_back
template <typename T, typename U>
void STLExtend(T* destination, const U& source) {
    typename U::const_iterator i = source.begin();
    const typename U::const_iterator end = source.end();
    for (; i != end; ++i) {
        destination->push_back(*i);
    }
}

namespace base {

/** Returns true if value is in container. */
template <typename ContainerType, typename ValueType>
bool contains(const ContainerType& container, const ValueType& value) {
    return std::find(container.begin(), container.end(), value) != container.end();
}

}

#endif
