/* AUTOMATICALLY GENERATED: DO NOT EDIT */
#ifndef DTXN_MESSAGES_H
#define DTXN_MESSAGES_H

#include <cassert>
#include <string>
#include <vector>

#include "base/assert.h"
#include "io/message.h"
#include "serialization.h"

namespace io {
class FIFOBuffer;
}

namespace dtxn {

// Contains a transaction fragment to be processed by the execution engine.
class Fragment : public io::Message {
public:
    Fragment() :
        id(0),
        multiple_partitions(false),
        last_fragment(true) {}

    // Unique identifier for this transaction. Used to identify messages that belong to a single transaction.
    int32_t id;

    // True if this transaction spans multiple partitions.
    bool multiple_partitions;

    // True if this is the last fragment of this transaction for this partition.
    bool last_fragment;

    // The actual transaction to be processed.
    std::string transaction;

    bool operator==(const Fragment& other) const {
        if (id != other.id) return false;
        if (multiple_partitions != other.multiple_partitions) return false;
        if (last_fragment != other.last_fragment) return false;
        if (transaction != other.transaction) return false;
        return true;
    }
    bool operator!=(const Fragment& other) const { return !(*this == other); }

    void appendToString(std::string* _out_) const {
        serialization::serialize(id, _out_);
        serialization::serialize(multiple_partitions, _out_);
        serialization::serialize(last_fragment, _out_);
        serialization::serialize(transaction, _out_);
    }

    virtual void serialize(io::FIFOBuffer* _out_) const {
        serialization::serialize(id, _out_);
        serialization::serialize(multiple_partitions, _out_);
        serialization::serialize(last_fragment, _out_);
        serialization::serialize(transaction, _out_);
    }

    const char* parseFromString(const char* _start_, const char* _end_) {
        _start_ = serialization::deserialize(&id, _start_, _end_);
        _start_ = serialization::deserialize(&multiple_partitions, _start_, _end_);
        _start_ = serialization::deserialize(&last_fragment, _start_, _end_);
        _start_ = serialization::deserialize(&transaction, _start_, _end_);
        return _start_;
    }

    void parseFromString(const std::string& _str_) {
        const char* end = parseFromString(_str_.data(), _str_.data() + _str_.size());
        ASSERT(end == _str_.data() + _str_.size());
    }

    static int32_t typeCode() { return 628009317; }
};

// The response from the partition.
class FragmentResponse : public io::Message {
public:
    FragmentResponse() :
        id(0),
        status(-1) {}

    // Identifies the transaction that this decision refers to.
    int32_t id;

    // Contains the ExecutionEngine::Status for this transaction.
    int32_t status;

    // The result of the transaction.
    std::string result;

    bool operator==(const FragmentResponse& other) const {
        if (id != other.id) return false;
        if (status != other.status) return false;
        if (result != other.result) return false;
        return true;
    }
    bool operator!=(const FragmentResponse& other) const { return !(*this == other); }

    void appendToString(std::string* _out_) const {
        serialization::serialize(id, _out_);
        serialization::serialize(status, _out_);
        serialization::serialize(result, _out_);
    }

    virtual void serialize(io::FIFOBuffer* _out_) const {
        serialization::serialize(id, _out_);
        serialization::serialize(status, _out_);
        serialization::serialize(result, _out_);
    }

    const char* parseFromString(const char* _start_, const char* _end_) {
        _start_ = serialization::deserialize(&id, _start_, _end_);
        _start_ = serialization::deserialize(&status, _start_, _end_);
        _start_ = serialization::deserialize(&result, _start_, _end_);
        return _start_;
    }

    void parseFromString(const std::string& _str_) {
        const char* end = parseFromString(_str_.data(), _str_.data() + _str_.size());
        ASSERT(end == _str_.data() + _str_.size());
    }

    static int32_t typeCode() { return -322395948; }
};

// Instructs a replica to commit or abort a transaction.
class CommitDecision : public io::Message {
public:
    CommitDecision() :
        id(0),
        commit(false) {}

    // Identifies the transaction that this decision refers to.
    int32_t id;

    // True if the transaction should commit.
    bool commit;

    bool operator==(const CommitDecision& other) const {
        if (id != other.id) return false;
        if (commit != other.commit) return false;
        return true;
    }
    bool operator!=(const CommitDecision& other) const { return !(*this == other); }

    void appendToString(std::string* _out_) const {
        serialization::serialize(id, _out_);
        serialization::serialize(commit, _out_);
    }

    virtual void serialize(io::FIFOBuffer* _out_) const {
        serialization::serialize(id, _out_);
        serialization::serialize(commit, _out_);
    }

    const char* parseFromString(const char* _start_, const char* _end_) {
        _start_ = serialization::deserialize(&id, _start_, _end_);
        _start_ = serialization::deserialize(&commit, _start_, _end_);
        return _start_;
    }

    void parseFromString(const std::string& _str_) {
        const char* end = parseFromString(_str_.data(), _str_.data() + _str_.size());
        ASSERT(end == _str_.data() + _str_.size());
    }

    static int32_t typeCode() { return -743738311; }
};

// Transaction record stored in a log.
class LogEntry : public io::Message {
public:
    LogEntry() :
        multiple_partitions(false),
        decision_log_entry(-1),
        commit(false) {}

    // All fragments for the transaction collected together.
    std::vector<std::string> fragments;

    // True if this transaction spans multiple partitions.
    bool multiple_partitions;

    // If this is a decision, this is the log entry for the transaction.
    int32_t decision_log_entry;

    // If this is a decision, True if the transaction should be committed.
    bool commit;

    bool operator==(const LogEntry& other) const {
        if (fragments != other.fragments) return false;
        if (multiple_partitions != other.multiple_partitions) return false;
        if (decision_log_entry != other.decision_log_entry) return false;
        if (commit != other.commit) return false;
        return true;
    }
    bool operator!=(const LogEntry& other) const { return !(*this == other); }

    void appendToString(std::string* _out_) const {
        {
            int32_t _size_ = static_cast<int32_t>(fragments.size());
            assert(0 <= _size_ && static_cast<size_t>(_size_) == fragments.size());
            serialization::serialize(_size_, _out_);
            for (int _i_ = 0; _i_ < _size_; ++_i_) {
                serialization::serialize(fragments[_i_], _out_);
            }
        }
        serialization::serialize(multiple_partitions, _out_);
        serialization::serialize(decision_log_entry, _out_);
        serialization::serialize(commit, _out_);
    }

    virtual void serialize(io::FIFOBuffer* _out_) const {
        {
            int32_t _size_ = static_cast<int32_t>(fragments.size());
            assert(0 <= _size_ && static_cast<size_t>(_size_) == fragments.size());
            serialization::serialize(_size_, _out_);
            for (int _i_ = 0; _i_ < _size_; ++_i_) {
                serialization::serialize(fragments[_i_], _out_);
            }
        }
        serialization::serialize(multiple_partitions, _out_);
        serialization::serialize(decision_log_entry, _out_);
        serialization::serialize(commit, _out_);
    }

    const char* parseFromString(const char* _start_, const char* _end_) {
        {
            int32_t _size_;
            _start_ = serialization::deserialize(&_size_, _start_, _end_);
            assert(_size_ >= 0);
            fragments.resize(_size_);
            for (int _i_ = 0; _i_ < _size_; ++_i_) {
                _start_ = serialization::deserialize(&fragments[_i_], _start_, _end_);
            }
        }
        _start_ = serialization::deserialize(&multiple_partitions, _start_, _end_);
        _start_ = serialization::deserialize(&decision_log_entry, _start_, _end_);
        _start_ = serialization::deserialize(&commit, _start_, _end_);
        return _start_;
    }

    void parseFromString(const std::string& _str_) {
        const char* end = parseFromString(_str_.data(), _str_.data() + _str_.size());
        ASSERT(end == _str_.data() + _str_.size());
    }

    static int32_t typeCode() { return -477062385; }
};

}  // namespace dtxn
#endif  // DTXN_MESSAGES_H
