#ifndef DTXN_ORDEREDDISTRIBUTOR_H
#define DTXN_ORDEREDDISTRIBUTOR_H

#include <vector>

#include "base/slidingarray.h"

namespace io {
class EventLoop;
}

namespace net {
class ConnectionHandle;
class MessageServer;
}

namespace dtxn {

class Coordinator;
class Fragment;
class FragmentResponse;
class Partition;

class OrderedDistributor {
public:
    // Owns partitions. Does not delete coordinator, event_loop or msg_server.
    OrderedDistributor(const std::vector<net::ConnectionHandle*>& partitions,
            Coordinator* coordinator, io::EventLoop* event_loop, net::MessageServer* msg_server);

    ~OrderedDistributor();

    // INTERNAL: request has been received from connection.
    void requestReceived(net::ConnectionHandle* connection, const Fragment& request);

    // INTERNAL: response has been received from connection.
    void responseReceived(net::ConnectionHandle* connection, const FragmentResponse& response);

    io::EventLoop* event_loop() { return event_loop_; }

private:
    class TransactionState;

    // Called when we have not received all the responses for transaction.
    void responseTimeout(TransactionState* transaction);

    void dispatchTransaction(TransactionState* transaction);

    // Processes the round of messages returned by the coordinator.
    void processFragments(TransactionState* transaction);

    // Sends the decision to all partitions and sends the response to the client
    void finishTransaction(TransactionState* transaction);

    std::vector<net::ConnectionHandle*> partitions_;
    Coordinator* coordinator_;

    typedef base::SlidingArray<TransactionState*> QueueType;
    QueueType queue_;

    io::EventLoop* event_loop_;
    net::MessageServer* msg_server_;
};

}  // namespace dtxn

#endif
