#ifndef MINITRANSACTIONDISTRIBUTOR_H__
#define MINITRANSACTIONDISTRIBUTOR_H__

#include <string>

#include "dtxn/dtxndistributor.h"

namespace htable {
class MiniResult;
}

class MiniTransactionDistributor : public dtxn::Coordinator {
public:
    MiniTransactionDistributor(int num_logical_partitions, int num_physical_partitions) :
            num_logical_partitions_(num_logical_partitions),
            num_physical_partitions_(num_physical_partitions),
            use_two_round_variant_(false) {
        assert(num_logical_partitions_ > 0);
        assert(num_physical_partitions_ > 0);
        assert(num_logical_partitions_ % num_physical_partitions_ == 0);
    }

    virtual void begin(dtxn::CoordinatedTransaction* transaction);
    virtual void nextRound(dtxn::CoordinatedTransaction* transaction);
    virtual void done(dtxn::CoordinatedTransaction* transaction);

    int partitionFunction(const std::string& key) const;

    static const char RESET_MAXIMUM_KEY[];

    void set_two_round(bool two_round) { use_two_round_variant_ = two_round; }

private:
    int num_logical_partitions_;
    int num_physical_partitions_;

    // If true, use the two round transaction variant. Used for "general transaction" tests.
    bool use_two_round_variant_;
};

#endif
