#include "minitransactions/minitransactions.h"

#include <cassert>

#include "base/stlutil.h"

using std::string;
using std::pair;

namespace minitransactions {

MiniTransaction::~MiniTransaction() {
    STLDeleteValues(&write_set_);
    STLDeleteValues(&compare_set_);
}

void MiniTransaction::read(const string& variable) {
    read_set_.insert(variable);
}

static void replacePointer(KeyValueSet* map, const string& variable, string* valueptr) {
    pair<KeyValueSet::iterator, bool> result =
            map->insert(make_pair(variable, valueptr));
    if (!result.second) {
        // Did not insert: key already exists: delete any value and replace
        delete result.first->second;
        result.first->second = valueptr;
    }
    assert((*map)[variable] == valueptr);
}

void MiniTransaction::write(const string& variable, const string& value) {
    replacePointer(&write_set_, variable, new string(value));
}

void MiniTransaction::erase(const string& variable) {
    replacePointer(&write_set_, variable, NULL);
}

void MiniTransaction::compare(const string& variable, const string* value) {
    string* valueptr = NULL;
    if (value != NULL) {
        valueptr = new string(*value);
    }
    replacePointer(&compare_set_, variable, valueptr);
}

}  // namespace minitransactions
