#!/usr/bin/python

"""Data Extraction Tools

Ideally this module will be a set of reusable tools for extracting and summarizing data stored in
CSV files. Right now it is very crappy and unorganized."""

import csv
import sqlite3

import plot_line_average
import stupidplot

def tryFloat(s):
    try:
        return float(s)
    except ValueError:
        return s

def readCSVTable(path):
    """Returns a 2D list from the csv file in path."""

    data = open(path)
    reader = csv.reader(data)
    table = []
    for row in reader:
        table.append([tryFloat(s) for s in row])
    data.close()

    return table


def expandRows(table, key_columns, value_columns):
    """Expands a row with multiple data points into one row per data point."""

    out = []
    for in_row in table:
        # Pull the key columns from in_row into a template
        out_row_template = []
        for key_column in key_columns:
            out_row_template.append(in_row[key_column])

        # For each value, output a row with (key, value)
        for value_column in value_columns:
            out_row = list(out_row_template)
            out_row.append(in_row[value_column])
            out.append(out_row)

    return out


def groupBy(table, group_columns):
    groups = {}
    for row in table:
        key = []
        for column in group_columns:
            key.append(row[column])
        key = tuple(key)

        if key not in groups:
            groups[key] = []
        groups[key].append(row)
    return groups


def select(table, equalityConditions):
    out_table = []
    for row in table:
        match = True
        for column, value in equalityConditions:
            if row[column] != value:
                match = False
                break
        if match:
            out_table.append(row)
    return out_table


def unique(table, column):
    """Returns a list of unique values in column."""

    keys = groupBy(table, [column]).keys()
    keys.sort()
    return keys


def selectStats(table, group_columns, value_column):
    """Stats are (average, median, standard deviation, min, max, 95% confidence interval)"""

    groups = groupBy(table, group_columns)

    out = []
    keys = groups.keys()
    keys.sort()
    for key in keys:
        values = []
        for row in groups[key]:
            values.append(row[value_column])

        stats = stupidplot.stats(values)
        out.append(key + stats)

    return out


def foo(path):
    # Read in the table
    table = readCSVTable(path)

    # Expand each repeated data point
    table = expandRows(table, [0, 1], [2, 3, 4, 5, 6])
    table.insert(0, ["conflict_percent", "multipartition_percent", "txns_per_sec"])

    connection = sqlite3.connect(":memory:")
    c = connection.cursor()

    # Create the table
    c.execute("create table results (%s);" % (", ".join(table[0])))

    # Import the data
    insert_template = "?," * len(table[0])
    insert_template = "insert into results values (%s)" % (insert_template[:-1])
    print insert_template
    for row in table[1:]:
        c.execute(insert_template, row)

    datasets = []
    for v in list(c.execute("select distinct conflict_percent from results")):
        conflict_percent = v[0]
        #~ legends.append("Lock %.0f%% conflict" % (conflict_percent))
        table = [["% Multipartition", "Lock %.0f%% conflict" % (conflict_percent), "- conf", "+ conf"]]
        for y in list(c.execute("select distinct multipartition_percent from results where "
                "conflict_percent = ? order by multipartition_percent asc", [conflict_percent])):
            y = y[0]
            x_values = []
            for x in c.execute("select txns_per_sec from results where conflict_percent = ? and multipartition_percent = ?", (conflict_percent, y)):
                x_values.append(x[0])
        
            stats = stupidplot.stats(x_values)
            table.append([y, stats[0], stats[0]-stats[-1], stats[0]+stats[-1]])
            print table[-1]
        datasets.append(table)
    
    return datasets


if __name__ == "__main__":
    import sys

    datasets = foo(sys.argv[1])
    plot_line_average.plotAverage(datasets, sys.argv[2])
