#!/usr/bin/python

import csv
import os.path

import stupidplot

def extractTables(path, x_axis_title, y_axis_title, line_title):
    """Returns two tables containing the values from the CSV file in path. The first contains the
    the columns (x, average, -95% confidence, +95% confidence). The second contains (x, y)
    scatter plot values."""

    data = open(path)
    reader = csv.reader(data)
    table = []
    for row in reader:
        table.append(row)
    data.close()

    scatter = [(x_axis_title, y_axis_title)]
    average = [(x_axis_title, line_title + " Average", "-95% confidence", "+95% confidence")]
    for row in table:
        x = int(row[0])
        for y in row[1:]:
            scatter.append((x, y))
        stats = stupidplot.stats([float(f) for f in row[1:]])
        average.append([x, stats[0], stats[0]-stats[-1], stats[0]+stats[-1]])

    return average, scatter


def plotScatter(scatter_data, output_path):
    options = {
        'plottype': 'points',
        'key': False,
        #~ 'ylabel': scatter_block[0][1],
        'yformat': '%g',
        'xformat': '%g',
    }
    stupidplot.gnuplotTable(scatter_data, output_path, options)


def plotAverage(average_data, output_path):
    options = {
        #~ 'plottype': 'points',
        #~ 'key': False,
        #~ 'ylabel': 'Transactions/s',
        'errorbars': [1],
        'yformat': '%g',
        'xformat': '%g',
        #~ 'xrange' : "[0:]",
        'yrange' : "[0:]",
    }
    stupidplot.gnuplotTable(average_data, output_path, options)


if __name__ == "__main__":
    import sys

    if len(sys.argv) == 2:
        average, scatter = extractTables(sys.argv[1], "% Multi-Partition Transactions", "Transactions/second", sys.argv[1])
        for row in average:
            print "\t".join((str(v) for v in row))
        sys.exit(0)

    if len(sys.argv) < 3:
        sys.stderr.write("plot_line_average.py [input files] [output.eps]\n")
        sys.exit(1)
    input_files = sys.argv[1:-1]
    output_file = sys.argv[-1]

    if os.path.exists(output_file):
        sys.stderr.write("output file %s exists: refusing to overwrite\n" % (sys.argv[-1]))
        sys.exit(1)

    averages = []
    for in_file in input_files:
        average, scatter = extractTables(in_file, "% Multi-Partition Transactions", "Transactions/second", in_file)
        averages.append(average)

    plotAverage(averages, output_file)
