#!/usr/bin/python

"""Tests HTable performance while scaling the % of multi-partition transactions."""

import csv
import os.path
import sys

from third_party.pystdlib import subprocess

PERF = "./htableclientperf"

TRIALS = 3
MAX_PARTITIONS = 2
THREADS = 2

def clientPerf(distributor, configuration_file, dispatch, threads, percent_multipartition):
    child = subprocess.Popen((PERF, distributor, configuration_file, str(dispatch),
            str(threads), str(percent_multipartition), "0"), stdout=subprocess.PIPE)
    output = child.stdout.read()
    result = child.wait()
    assert result == 0

    return float(output.strip())


def saveCSVResults(configuration_file, results):
    output_path = os.path.splitext(configuration_file)[0] + ".csv"
    out = open(output_path, "wb")
    writer = csv.writer(out)
    writer.writerows(results)
    out.close()


def runTest(configuration_file, distributor):
    results = []

    partitions = MAX_PARTITIONS
    for percent_multipartition in range(0, 20, 2) + range(20, 101, 5):
        # 20 transactions per partition
        dispatch = partitions * 20
        threads = dispatch

        row = [percent_multipartition]
        print "%d%% multipartition" % row[0], 
        sys.stdout.flush()
        for _ in range(TRIALS):
            output = clientPerf(
                    distributor, configuration_file, dispatch, threads, percent_multipartition)
            row.append(output)
            print "\t", row[-1],
            sys.stdout.flush()
        results.append(row)
        print

    saveCSVResults(configuration_file, results)


if __name__ == "__main__":
    if len(sys.argv) != 3:
        sys.stderr.write("scale_mptxns.py [config file] [distributor]\n")
        sys.exit(1)

    configuration_file = sys.argv[1]
    distributor = sys.argv[2]

    runTest(configuration_file, distributor)
