#!/usr/bin/python

"""
Configurations TODO:

* client -> 1 server: scale number of pending txns
* client -> distributor -> 1 server: 100% SP, scale number of pending txns
* client -> distributor -> 1 server: mix of SP and "multi-partition" (shows "overhead")
* client -> distributor -> 2 servers, 1 core: shows messaging overhead
* client -> distributor -> 2 servers: scale % MP txns
"""

import csv
import os.path
import subprocess
import sys

PERF = "./htableclientperf-lock-conflict"

TRIALS = 5
PARTITIONS = 2
# 10 simultaneous transactions per partition
TRANSACTIONS_PER_PARTITION = 15

def runTest(configuration_file):
    results = []

    for percent_conflict in range(0, 101, 20):
        for percent_multipartition in range(0, 20, 2) + range(20, 101, 5):
            clients = PARTITIONS * TRANSACTIONS_PER_PARTITION 

            row = [percent_conflict, percent_multipartition]
            print "%d%% conflict %d%% multipartition" % (row[0], row[1])
            sys.stdout.flush()
            for _ in range(TRIALS):
                child = subprocess.Popen(
                        (PERF, configuration_file, str(clients), str(percent_multipartition), str(percent_conflict)),
                        stdout=subprocess.PIPE)
                output = child.stdout.read()
                result = child.wait()
                assert result == 0

                output = float(output.strip())
                row.append(output)
                print "\t", row[-1],
                sys.stdout.flush()
            results.append(row)
            print

    output_path = os.path.splitext(configuration_file)[0] + ".csv"
    out = open(output_path, "wb")
    writer = csv.writer(out)
    writer.writerows(results)
    out.close()

if __name__ == "__main__":
    if len(sys.argv) != 2:
        sys.stderr.write("scale_mptxns_conflict.py [config file]\n")
        sys.exit(1)

    configuration_file = sys.argv[1]

    runTest(configuration_file)
