#include "randomgenerator.h"

#include <algorithm>
#include <cstdlib>
#include <cstring>
#include <ctime>

#include "base/assert.h"

RandomGenerator::RandomGenerator() {
#ifdef HAVE_RANDOM_R
    // Set the random state to zeros. glibc will attempt to access the old state if not NULL.
    memset(&state, 0, sizeof(state));
    int result = initstate_r(static_cast<unsigned int>(time(NULL)), state_array,
            sizeof(state_array), &state);
    ASSERT(result == 0);
#else
    seed(time(NULL));
#endif
}

int32_t RandomGenerator::random() {
    int32_t rand_int;
#ifdef HAVE_RANDOM_R
    int error = random_r(&state, &rand_int);
    ASSERT(error == 0);
#else
    rand_int = nrand48(state);
#endif
    ASSERT(0 <= rand_int && rand_int <= RAND_MAX);
    return rand_int;
}

void RandomGenerator::seed(unsigned int value) {
#ifdef HAVE_RANDOM_R
    int error = srandom_r(value, &state);
    ASSERT(error == 0);
#else
    memcpy(state, &value, std::min(sizeof(value), sizeof(state)));
#endif
}
