#ifndef RANDOMGENERATOR_H__
#define RANDOMGENERATOR_H__

#include <cstdlib>  // for struct random_data

#ifdef __linux__
#define HAVE_RANDOM_R
#endif

// A random number generator that uses random_r.
class RandomGenerator {
public:
    // Seeds the generator with the current time.
    RandomGenerator();

    // Returns a pseudo-random number from 0 to maximum().
    int32_t random();

    // Returns the maximum value that can be generated by random() (inclusive).
    static int32_t maximum() { return RAND_MAX; }

    // Seed the generator with seed.
    void seed(unsigned int seed);

private:
#ifdef HAVE_RANDOM_R
    // man random says optimal sizes are 8, 32, 64, 128, 256 bytes
    static const int RANDOM_STATE_SIZE = 64;
    char state_array[RANDOM_STATE_SIZE];
    struct random_data state;
#else
    unsigned short state[3];
#endif
};

#endif
