#ifndef REPLICATION_CACHEDSLIDINGARRAY_H__
#define REPLICATION_CACHEDSLIDINGARRAY_H__

#include <vector>

#include "base/slidingarray.h"
#include "base/stlutil.h"

namespace replication {

// T must support .clear() and default construction.
// TODO: We could reuse the elements in an underlying array. It would make resizing more expensive
// though, since we would need to copy all the elements rather than pointers.
template <typename T>
class CachedSlidingArray {
public:
    CachedSlidingArray() {}
    ~CachedSlidingArray() {
        STLDeleteElements(&array_);
        STLDeleteElements(&recycler_);
    }

    // Allocate or reuse a T, at the end of the buffer.
    T* add() {
        T* v = NULL;
        if (!recycler_.empty()) {
            v = recycler_.back();
            recycler_.pop_back();
        } else {
            v = new T();
        }

        array_.push_back(v);
        return v;
    }

    bool empty() const { return array_.empty(); }
    size_t size() const { return array_.size(); }
    size_t firstIndex() const { return array_.firstIndex(); }
    size_t nextIndex() const { return array_.nextIndex(); }
    const T& at(size_t index) const { return *array_.at(index); }
    const T& front() const { return *array_.front(); }

    void pop_front() {
        // Fetch and remove the first element
        T* v = array_.dequeue();

        // Hold on to the element in the recycle list
        v->clear();
        recycler_.push_back(v);
    }

    T* mutableAt(size_t index) { return array_.at(index); }
    
private:
    // The underlying array.
    base::SlidingArray<T*> array_;

    // Stores allocated objects to be recycled.
    std::vector<T*> recycler_;

    CachedSlidingArray(const CachedSlidingArray& other);
    CachedSlidingArray& operator=(const CachedSlidingArray& other);
};

}  // namespace replication
#endif
