#ifndef TPCCCOORDINATOR_H__
#define TPCCCOORDINATOR_H__

#include <cassert>
//~ #include <cstdlib>
//~ #include <tr1/unordered_set>

//~ #include "base/cast.h"
//~ #include "bench/tpccdb.h"
//~ #include "dtxn/common.h"
//~ #include "dtxn/configparser.h"
#include "dtxn/dtxndistributor.h"
//~ #include "serialization.h"
//~ #include "tpccengine.h"

class TPCCCoordinator : public dtxn::Coordinator {
public:
    TPCCCoordinator(int num_warehouses, int num_partitions) :
            num_warehouses_(num_warehouses) {
        assert(num_warehouses_ > 0);
        assert(num_warehouses >= num_partitions);
        assert(num_warehouses_ % num_partitions == 0);
        warehouses_per_partition_ = num_warehouses_ / num_partitions;
    }

    virtual void begin(dtxn::CoordinatedTransaction* transaction);
    virtual void nextRound(dtxn::CoordinatedTransaction* transaction);
    virtual void done(dtxn::CoordinatedTransaction* transaction);

private:
    // The partition function assumes that warehouses are evenly distributed
    // TODO: Reduce duplication with tpccclientperf
    int warehouseToPartition(int32_t warehouse_id) {
        assert(warehouse_id <= num_warehouses_);
        int client_index = (warehouse_id - 1) / warehouses_per_partition_;
        return client_index;
    }

    void beginNewOrder(dtxn::CoordinatedTransaction* transaction);
    void finishNewOrder(dtxn::CoordinatedTransaction* transaction);
    void beginPayment(dtxn::CoordinatedTransaction* transaction);
    void finishPayment(dtxn::CoordinatedTransaction* transaction);
    void beginReset(dtxn::CoordinatedTransaction* transaction);
    void finishReset(dtxn::CoordinatedTransaction* transaction);

    int num_warehouses_;
    int warehouses_per_partition_;
};

#endif
