// Copyright 2008,2009,2010 Massachusetts Institute of Technology.
// All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

#define __STDC_FORMAT_MACROS
#include <cstdio>
#include <inttypes.h>

#include "bench/clock.h"
#include "bench/randomgenerator.h"
#include "bench/tpccclient.h"
#include "bench/tpccgenerator.h"
#include "bench/tpcctables.h"

#ifdef PROFILE
#include "google/profiler.h"
#else
static void ProfilerStart(const char* filename) __attribute__((unused));
static void ProfilerStart(const char* filename) {}
static void ProfilerStop() __attribute__((unused));
static void ProfilerStop() {}
#endif

static const int NUM_WAREHOUSES = 3;
static const int TRIALS = 5;
static const int TXNS_PER_TRIAL = 100000;

int main(int argc, const char* argv[]) {
    TPCCTables* tables = new TPCCTables();
    SystemClock* clock = new SystemClock();

    // Create a generator for filling the database.
    // TODO: seed the generator with a consistent seed
    tpcc::RealRandomGenerator* random = new tpcc::RealRandomGenerator();
    random->seed(0);
    tpcc::NURandC cLoad = tpcc::NURandC::makeRandom(random);
    random->setC(cLoad);
    char now[Clock::DATETIME_SIZE+1];
    clock->getDateTimestamp(now);
    TPCCGenerator generator(random, now, Item::NUM_ITEMS, District::NUM_PER_WAREHOUSE,
            Customer::NUM_PER_DISTRICT, NewOrder::INITIAL_NUM_PER_DISTRICT);

    // Owns clock, generator, tables.
    random = new tpcc::RealRandomGenerator();
    tpcc::NURandC cRun = tpcc::NURandC::makeRandomForRun(random, cLoad);
    random->setC(cRun);
    TPCCClient client(clock, random, tables, Item::NUM_ITEMS, NUM_WAREHOUSES,
            District::NUM_PER_WAREHOUSE, Customer::NUM_PER_DISTRICT);

    // Generate the data
    generator.makeItemsTable(tables);
    for (int i = 1; i <= NUM_WAREHOUSES; ++i) {
        generator.makeWarehouse(tables, i);
    }

    int64_t total_us = 0;
    printf("%d warehouses:", NUM_WAREHOUSES);
    fflush(stdout);
    ProfilerStart("profile.out");
    for (int j = 0; j < TRIALS; ++j) {
        int64_t begin = clock->getMicroseconds();
        for (int k = 0; k < TXNS_PER_TRIAL; ++k) {
            client.doNewOrder();
        }
        int64_t end = clock->getMicroseconds();
        printf(" %"PRId64, (end-begin)/1000);
        total_us += (end-begin);
        fflush(stdout);
    }
    ProfilerStop();
    printf(" avg: %"PRId64" ms %.3f us/txn\n", total_us/1000/TRIALS, ((double)total_us)/TRIALS/TXNS_PER_TRIAL);

    return 0;
}
