// Copyright 2008,2009,2010 Massachusetts Institute of Technology.
// All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

/* AUTOMATICALLY GENERATED: DO NOT EDIT */
#ifndef HTABLE_MESSAGES_H
#define HTABLE_MESSAGES_H

#include <cassert>
#include <string>
#include <vector>

#include "base/assert.h"
#include "io/message.h"
#include "serialization.h"

namespace io {
class FIFOBuffer;
}

namespace htable {

// Contains a key and a value.
class KeyValuePair : public io::Message {
public:
    KeyValuePair() :
        hasValue(false) {}

    // Key name.
    std::string key;

    // True if this pair contains a value.
    bool hasValue;

    // Value for the key.
    std::string value;

    bool operator==(const KeyValuePair& other) const {
        if (key != other.key) return false;
        if (hasValue != other.hasValue) return false;
        if (value != other.value) return false;
        return true;
    }
    bool operator!=(const KeyValuePair& other) const { return !(*this == other); }

    void appendToString(std::string* _out_) const {
        serialization::serialize(key, _out_);
        serialization::serialize(hasValue, _out_);
        serialization::serialize(value, _out_);
    }

    virtual void serialize(io::FIFOBuffer* _out_) const {
        serialization::serialize(key, _out_);
        serialization::serialize(hasValue, _out_);
        serialization::serialize(value, _out_);
    }

    const char* parseFromString(const char* _start_, const char* _end_) {
        _start_ = serialization::deserialize(&key, _start_, _end_);
        _start_ = serialization::deserialize(&hasValue, _start_, _end_);
        _start_ = serialization::deserialize(&value, _start_, _end_);
        return _start_;
    }

    void parseFromString(const std::string& _str_) {
        const char* end = parseFromString(_str_.data(), _str_.data() + _str_.size());
        ASSERT(end == _str_.data() + _str_.size());
    }

    static int32_t typeCode() { return -971916225; }
};

// A mini-transaction.
class MiniTransaction : public io::Message {
public:
    MiniTransaction() {}

    // Key/Value pairs to compare.
    std::vector<KeyValuePair > compare_set;

    // Keys to read.
    std::vector<std::string > read_set;

    // Keys to write.
    std::vector<KeyValuePair > write_set;

    bool operator==(const MiniTransaction& other) const {
        if (compare_set != other.compare_set) return false;
        if (read_set != other.read_set) return false;
        if (write_set != other.write_set) return false;
        return true;
    }
    bool operator!=(const MiniTransaction& other) const { return !(*this == other); }

    void appendToString(std::string* _out_) const {
        {
            int32_t _size_ = static_cast<int32_t>(compare_set.size());
            assert(0 <= _size_ && static_cast<size_t>(_size_) == compare_set.size());
            serialization::serialize(_size_, _out_);
            for (int _i0_ = 0; _i0_ < _size_; ++_i0_) {
                compare_set[_i0_].appendToString(_out_);
            }
        }
        {
            int32_t _size_ = static_cast<int32_t>(read_set.size());
            assert(0 <= _size_ && static_cast<size_t>(_size_) == read_set.size());
            serialization::serialize(_size_, _out_);
            for (int _i1_ = 0; _i1_ < _size_; ++_i1_) {
                serialization::serialize(read_set[_i1_], _out_);
            }
        }
        {
            int32_t _size_ = static_cast<int32_t>(write_set.size());
            assert(0 <= _size_ && static_cast<size_t>(_size_) == write_set.size());
            serialization::serialize(_size_, _out_);
            for (int _i2_ = 0; _i2_ < _size_; ++_i2_) {
                write_set[_i2_].appendToString(_out_);
            }
        }
    }

    virtual void serialize(io::FIFOBuffer* _out_) const {
        {
            int32_t _size_ = static_cast<int32_t>(compare_set.size());
            assert(0 <= _size_ && static_cast<size_t>(_size_) == compare_set.size());
            serialization::serialize(_size_, _out_);
            for (int _i3_ = 0; _i3_ < _size_; ++_i3_) {
                compare_set[_i3_].serialize(_out_);
            }
        }
        {
            int32_t _size_ = static_cast<int32_t>(read_set.size());
            assert(0 <= _size_ && static_cast<size_t>(_size_) == read_set.size());
            serialization::serialize(_size_, _out_);
            for (int _i4_ = 0; _i4_ < _size_; ++_i4_) {
                serialization::serialize(read_set[_i4_], _out_);
            }
        }
        {
            int32_t _size_ = static_cast<int32_t>(write_set.size());
            assert(0 <= _size_ && static_cast<size_t>(_size_) == write_set.size());
            serialization::serialize(_size_, _out_);
            for (int _i5_ = 0; _i5_ < _size_; ++_i5_) {
                write_set[_i5_].serialize(_out_);
            }
        }
    }

    const char* parseFromString(const char* _start_, const char* _end_) {
        {
            int32_t _size_;
            _start_ = serialization::deserialize(&_size_, _start_, _end_);
            assert(_size_ >= 0);
            compare_set.resize(_size_);
            for (int _i6_ = 0; _i6_ < _size_; ++_i6_) {
                _start_ = compare_set[_i6_].parseFromString(_start_, _end_);
            }
        }
        {
            int32_t _size_;
            _start_ = serialization::deserialize(&_size_, _start_, _end_);
            assert(_size_ >= 0);
            read_set.resize(_size_);
            for (int _i7_ = 0; _i7_ < _size_; ++_i7_) {
                _start_ = serialization::deserialize(&read_set[_i7_], _start_, _end_);
            }
        }
        {
            int32_t _size_;
            _start_ = serialization::deserialize(&_size_, _start_, _end_);
            assert(_size_ >= 0);
            write_set.resize(_size_);
            for (int _i8_ = 0; _i8_ < _size_; ++_i8_) {
                _start_ = write_set[_i8_].parseFromString(_start_, _end_);
            }
        }
        return _start_;
    }

    void parseFromString(const std::string& _str_) {
        const char* end = parseFromString(_str_.data(), _str_.data() + _str_.size());
        ASSERT(end == _str_.data() + _str_.size());
    }

    static int32_t typeCode() { return -1130153967; }
};

// Result of executing a mini-transaction.
class MiniResult : public io::Message {
public:
    MiniResult() {}

    // Keys and values that were read.
    std::vector<KeyValuePair > keyvalues;

    bool operator==(const MiniResult& other) const {
        if (keyvalues != other.keyvalues) return false;
        return true;
    }
    bool operator!=(const MiniResult& other) const { return !(*this == other); }

    void appendToString(std::string* _out_) const {
        {
            int32_t _size_ = static_cast<int32_t>(keyvalues.size());
            assert(0 <= _size_ && static_cast<size_t>(_size_) == keyvalues.size());
            serialization::serialize(_size_, _out_);
            for (int _i9_ = 0; _i9_ < _size_; ++_i9_) {
                keyvalues[_i9_].appendToString(_out_);
            }
        }
    }

    virtual void serialize(io::FIFOBuffer* _out_) const {
        {
            int32_t _size_ = static_cast<int32_t>(keyvalues.size());
            assert(0 <= _size_ && static_cast<size_t>(_size_) == keyvalues.size());
            serialization::serialize(_size_, _out_);
            for (int _i10_ = 0; _i10_ < _size_; ++_i10_) {
                keyvalues[_i10_].serialize(_out_);
            }
        }
    }

    const char* parseFromString(const char* _start_, const char* _end_) {
        {
            int32_t _size_;
            _start_ = serialization::deserialize(&_size_, _start_, _end_);
            assert(_size_ >= 0);
            keyvalues.resize(_size_);
            for (int _i11_ = 0; _i11_ < _size_; ++_i11_) {
                _start_ = keyvalues[_i11_].parseFromString(_start_, _end_);
            }
        }
        return _start_;
    }

    void parseFromString(const std::string& _str_) {
        const char* end = parseFromString(_str_.data(), _str_.data() + _str_.size());
        ASSERT(end == _str_.data() + _str_.size());
    }

    static int32_t typeCode() { return -1094725957; }
};

}  // namespace htable
#endif  // HTABLE_MESSAGES_H
