// Copyright 2008,2009,2010 Massachusetts Institute of Technology.
// All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

#include "htablebenchmark.h"
#include "htable/messages.h"
#include "stupidunit/stupidunit.h"

using std::vector;

using namespace htable;

class HTableBenchmarkDataTest : public Test {
public:
    HTableBenchmarkDataTest() : data_(12, 4) {}

    // Don't use DEFAULT_DATA because the test should not change if it changes
    DataDefinition data_;
};

TEST_F(HTableBenchmarkDataTest, BadDataDefinition) {
    EXPECT_DEATH(DataDefinition data(4, 5));
}

TEST_F(HTableBenchmarkDataTest, MakeSPTxn) {
    MiniTransaction mini = makeSPTransaction(data_, 0, 0);
    EXPECT_EQ(12, mini.read_set.size());
    EXPECT_EQ(12, mini.write_set.size());
}

TEST_F(HTableBenchmarkDataTest, MakeMPTxn) {
    MiniTransaction mini = makeMPTransaction(data_, 0);
    EXPECT_EQ(12, mini.read_set.size());
    EXPECT_EQ(12, mini.write_set.size());
}

TEST_F(HTableBenchmarkDataTest, PartitionFunction) {
    // bad number of physical partitions
    EXPECT_DEATH(partitionFunction(12, 5, 0));
    EXPECT_DEATH(partitionFunction(12, 24, 0));

    // bad partition ids
    EXPECT_DEATH(partitionFunction(12, 4, 12));
    EXPECT_DEATH(partitionFunction(12, 4, -1));

    EXPECT_EQ(0, data_.partitionFunction(4, 0));
    EXPECT_EQ(0, data_.partitionFunction(2, 0));
    EXPECT_EQ(0, data_.partitionFunction(1, 0));

    EXPECT_EQ(1, data_.partitionFunction(4, 1));
    EXPECT_EQ(0, data_.partitionFunction(2, 1));
    EXPECT_EQ(0, data_.partitionFunction(1, 1));

    EXPECT_EQ(3, data_.partitionFunction(4, 3));
    EXPECT_EQ(1, data_.partitionFunction(2, 3));
    EXPECT_EQ(0, data_.partitionFunction(1, 3));
}

TEST_F(HTableBenchmarkDataTest, MakeConflicts) {
    vector<MiniTransaction> v;
    v.push_back(makeSPTransaction(data_, 0, 0));
    
    vector<MiniTransaction> c = makeConflictSingles(v, 0);
    EXPECT_EQ(1, c.size());
    EXPECT_EQ(data_.num_variables_, c[0].write_set.size());
    // makeConflictSingles assumes the vector has one transaction per partition
    EXPECT_EQ(data_.num_variables_ - 1, c[0].read_set.size());
}

int main() {
    return TestSuite::globalInstance()->runAll();
}
