// Copyright 2008,2009,2010 Massachusetts Institute of Technology.
// All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

#include <cassert>
#include <cstdio>
#include <cstdlib>
#include <cstring>

#include "dtxn/common.h"
#include "dtxn/configparser.h"
#include "htablebenchmark.h"
#include "minitransactiondistributor.h"

static const char GENERAL_TXN_MODE[] = "general";

int main(int argc, const char* argv[]) {
    if (argc != 3 && argc != 4) {
        fprintf(stderr,
                "htabledistributor [listen port] [configuration file] [OPTIONAL: general]\n");
        return 1;
    }
    int port = atoi(argv[1]);
    assert(port != 0);
    const char* const config_file = argv[2];
    const char* const optional_mode = (argc >= 4) ? argv[3] : NULL;

    std::vector<dtxn::Partition> partitions = dtxn::parseConfigurationFromPath(config_file);
    MiniTransactionDistributor distributor(
            htable::DEFAULT_DATA.num_logical_partitions_, (int) partitions.size());
    if (optional_mode != NULL) {
        // Enable two round general transactions
        assert(strncmp(GENERAL_TXN_MODE, optional_mode, sizeof(GENERAL_TXN_MODE)) == 0);
        distributor.set_two_round(true);
    }
    dtxn::runDistributor(&distributor, port, config_file);

    return 0;
}
