// Copyright 2008,2009,2010 Massachusetts Institute of Technology.
// All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

#include <cstdio>
#include <cstdlib>

#include "dtxn/common.h"
#include "dtxn/configparser.h"
#include "dtxn/locks/lockscheduler.h"
#include "dtxn/occ/occscheduler.h"
#include "dtxn/ordered/orderedscheduler.h"
#include "htableengine.h"
#include "io/libeventloop.h"

using std::vector;

int main(int argc, const char* argv[]) {
    if (argc != 5) {
        fprintf(stderr, "htableserver [mode = block|spec|lock|occ] [configuration] [partition index] [replica index]\n");
        return 1;
    }
    const char* mode_string = argv[1];
    const char* config_file = argv[2];
    int partition_index = atoi(argv[3]);
    int replica_index = atoi(argv[4]);

    enum Mode {
        BLOCKING,
        SPECULATE,
        LOCKING,
        OCC,
    };
    Mode mode = LOCKING;
    if (strcmp(mode_string, "block") == 0) {
        mode = BLOCKING;
    } else if (strcmp(mode_string, "spec") == 0) {
        mode = SPECULATE;
    } else if (strcmp(mode_string, "lock") == 0) {
        mode = LOCKING;
    } else if (strcmp(mode_string, "occ") == 0) {
        mode = OCC;
    } else {
        fprintf(stderr, "mode must be block|spec|lock|occ\n");
        return 1;
    }

    vector<dtxn::Partition> partitions = dtxn::parseConfigurationFromPath(config_file);
    assert(0 <= partition_index && partition_index < partitions.size());
    assert(0 <= replica_index && replica_index < partitions[partition_index].numReplicas());

    htable::HTableEngine engine(mode == LOCKING || mode == OCC);

    dtxn::Scheduler* scheduler = NULL;
    if (mode == LOCKING) {
        //~ scheduler = new dtxn::LockScheduler(&engine);
    } else if (mode == OCC) {
        //~ scheduler = new dtxn::OCCScheduler(&engine);
    } else {
        assert(mode == BLOCKING || mode == SPECULATE);
        scheduler = new dtxn::OrderedScheduler(&engine);
        if (mode == SPECULATE) {
            ((dtxn::OrderedScheduler*) scheduler)->cc_mode(dtxn::OrderedScheduler::CC_SPECULATIVE);
        } else {
            assert(mode == BLOCKING);
        }
    }

    io::LibEventLoop event_loop;
    dtxn::runServer(&event_loop, scheduler, partitions[partition_index], replica_index);

    // Print the contents of the table
    engine.print();

    return 0;
}
