// Copyright 2008,2009,2010 Massachusetts Institute of Technology.
// All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

#ifndef MINITRANSACTIONS_LOCKINGKEYVALUETABLE_H__
#define MINITRANSACTIONS_LOCKINGKEYVALUETABLE_H__

#include <cassert>
#include <string>

#include "minitransactions/keyvalueengine.h"  // for KeyValueTable

namespace dtxn {
class Transaction;
}

namespace minitransactions {

/** An implementation of VariableTable that never blocks (no concurrency control). */
class LockingKeyValueTable : public KeyValueTable {
public:
    virtual ~LockingKeyValueTable();
    virtual bool read(dtxn::Transaction* transaction, const std::string& key,
            const std::string** output);
    virtual bool write(dtxn::Transaction* transaction, const std::string& key,
            const std::string* value, std::string** undo);
    virtual void print();

private:
    class LockingString;
    LockingString* getValue(const std::string& key);

    typedef std::tr1::unordered_map<std::string, LockingString*> TableType;
    TableType table_;
};

}  // namespace minitransactions

#endif
