// Copyright 2008,2009,2010 Massachusetts Institute of Technology.
// All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

#ifndef MINITRANSACTIONS_H__
#define MINITRANSACTIONS_H__

/** Implementation of a (key, value) table implementing "mini-transactions." */

#include <string>
#include <tr1/unordered_map>
#include <tr1/unordered_set>

#include "strings/utils.h"

namespace minitransactions {

typedef std::tr1::unordered_map<std::string, std::string*> KeyValueSet;

class MiniTransaction {
public:
    MiniTransaction() {}
    ~MiniTransaction();

    void read(const std::string& variable);
    void write(const std::string& variable, const std::string& value);
    void erase(const std::string& variable);
    void compare(const std::string& variable, const std::string* value);

    typedef std::tr1::unordered_set<std::string> ReadSet;

    const ReadSet& read_set() const { return read_set_; }
    const KeyValueSet& write_set() const { return write_set_; }
    const KeyValueSet& compare_set() const { return compare_set_; }

private:
    ReadSet read_set_;
    KeyValueSet write_set_;
    KeyValueSet compare_set_;

    // Disable copies
    MiniTransaction(const MiniTransaction& other);
};

}  // namespace minitransactions

#endif
