#!/usr/bin/python

import dataextract
import mptxn_plot
import stupidplot


#~ BLOCK = "data/abort-block.csv"
#~ LOCK = "data/abort-lock.csv"
SPEC = "data/abort-spec.csv"

def extract(path, type):
    # Read in the table
    table = dataextract.readCSVTable(path)

    # Expand each repeated data point
    table = dataextract.expandRows(table, [0, 1], [2, 3, 4])
    return dataextract.insertColumn(table, 0, type)


if __name__ == "__main__":
    output_path = mptxn_plot.get_output()

    # Keep only 10% from blocking and locking (best performance)
    # TODO: Fix this; but for now this takes way too long to run these tests
    #~ table = extract(BLOCK, "Blocking")
    #~ table.extend(extract(LOCK, "Locking"))    
    #~ table = dataextract.select(table, [(1, 10)])

    table = extract(SPEC, "Speculation")

    # Add the original results for 0% aborts
    for path, line_title in (mptxn_plot.INPUTS[1], mptxn_plot.INPUTS[2]):
        temp = dataextract.readCSVTable(path)
        temp = dataextract.insertColumn(temp, 0, 0)
        temp = dataextract.insertColumn(temp, 0, line_title)
        table.extend(dataextract.expandRows(temp, [0, 1, 2], [3, 4, 5]))

    # Keep only 0, 3, 5, 10% abort (dropping 1%)
    table = [row for row in table if row[1] == 10 or row[1] == 5 or row[1] == 3 or row[1] == 0]

    datasets = []
    groups = dataextract.groupBy(table, (0, 1))
    keys = groups.keys()
    def hackSort(a, b):
        """Custom sort for ordering lines for the graph."""
        ORDER = ["Speculation", "Blocking", "Locking"]
        a_label = ORDER.index(a[0])
        b_label = ORDER.index(b[0])
        v = cmp(a_label, b_label)
        if v != 0: return v
        return cmp(a[1], b[1])
    keys.sort(hackSort)
    for key in keys:
        data = dataextract.selectStatsConfPlot(groups[key], [2], 3)

        label = "%s %d%% aborts" % key
        data.insert(0, [mptxn_plot.X_AXIS, label, "- conf", "+ conf"])
        datasets.append(mptxn_plot.removeErrorBars(data))

    options = dict(mptxn_plot.DEFAULT_PLOT_OPTIONS)
    for i in range(1, 5):
        options["colors"][i] = mptxn_plot.BLACK
    options["colors"][5] = mptxn_plot.RED
    options["colors"][6] = mptxn_plot.BLUE
    options["errorbars"] = [1]
    del options["errorbars"]

    stupidplot.gnuplotTable(datasets, output_path, options)
