#!/usr/bin/python

"""Add a licence header to the specified files."""

import fnmatch
import os

LINE_LENGTH_LIMIT = 100

def recursiveGlob(start_path, pattern):
    if not os.path.isdir(start_path):
        if fnmatch.fnmatch(start_path, pattern):
            yield start_path
    else:
        stack = [start_path]

        while len(stack) > 0:
            dirpath = stack[-1]
            del stack[-1]
            for path in os.listdir(dirpath):
                path = os.path.join(dirpath, path)
                if os.path.isdir(path):
                    stack.append(path)
                elif fnmatch.fnmatch(path, pattern):
                    yield path

HEADER="""// Copyright 2008,2009,2010 Massachusetts Institute of Technology.
// All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.""".split("\n")
for i, line in enumerate(HEADER):
    HEADER[i] = line + "\n"
del i

def addHeader(path):
    f = open(path)
    lines = f.readlines()
    f.close()

    match = True
    for i, header_line in enumerate(HEADER):
        if lines[i] != header_line:
            match = False
            break
    if match:
        return False

    lines = list(HEADER) + ["\n"] + lines

    f = open(path, "w")
    f.write("".join(lines))
    f.close()
    return True


if __name__ == "__main__":
    EXCLUDE_LIST = ("libevent", "third_party")

    for source_extension in ("*.cc", "*.h", "*.java"):
        for source_file in recursiveGlob(".", source_extension):
            excluded = False
            for exclude in EXCLUDE_LIST:
                if source_file.startswith("./" + exclude):
                    excluded = True
                    break
            if excluded:
                continue

            if addHeader(source_file):
                print source_file
