#!/usr/bin/python

import csv
import math
import sys

#~ from third_party.pystdlib import subprocess
import subprocess

#~ PERF = "build/mysqlengine/traceclient"
#~ TRACE = "/home/evanj/t/tpcc-trace/47.sql"
PERF = "./traceclient"
TRACE = "combined.sql"


def stats(r):
    total = sum(r)
    average = total/float(len(r))
    sum_deviation_squared = sum(((i-average)**2 for i in r))
    standard_deviation = math.sqrt(sum_deviation_squared/(len(r)-1 or 1))
    s = list(r)
    s.sort()
    median = s[len(s)/2]
    minimum = s[0]
    maximum = s[-1]
    return average, median, standard_deviation, minimum, maximum


def clientPerf(distributor, desired_rate):
    child = subprocess.Popen((PERF, distributor, TRACE, str(desired_rate)), stdout=subprocess.PIPE)
    output = child.stdout.read()
    result = child.wait()
    assert result == 0

    lines = output.split("\n")
    output = None
    measured_rate = float(lines[0].split()[0])
    latencies = [int(x) for x in lines[1].split()]
    lines = None
    average, median, standard_deviation, minimum, maximum = stats(latencies)

    return desired_rate, measured_rate, average, median, standard_deviation, minimum, maximum


def saveCSVResults(results, output_path):
    out = open(output_path, "wb")
    writer = csv.writer(out)
    writer.writerows(results)
    out.close()


def runTest(distributor, output_path):
    results = []

    results.append(("Desired", "Measured", "Average", "Median", "Std. Dev.", "Min", "Max"))
    for rate in range(5, 301, 5):
        print "%d requests per second: " % rate,
        sys.stdout.flush()
        row = clientPerf(distributor, rate)
        results.append(row)
        print "%f measured requests per second; %d ms average latency" % (row[1], row[2])

    saveCSVResults(results, output_path)


if __name__ == "__main__":
    if len(sys.argv) != 3:
        sys.stderr.write("mysqlload.py [distributor] [output.csv]\n")
        sys.exit(1)

    distributor = sys.argv[1]
    output_path = sys.argv[2]

    runTest(distributor, output_path)
