#!/usr/bin/python

import csv
import sys

import stupidplot


def model(clients, one_client_txns_per_s, max_txns_per_s):
    assert max_txns_per_s > one_client_txns_per_s
    s_per_one_txn = 1.0 / one_client_txns_per_s
    s_per_txn_min = 1.0 / max_txns_per_s
    
    s_round_trip = s_per_one_txn - s_per_txn_min
    
    n_per_delay = s_round_trip / s_per_txn_min
    if clients < 1 + n_per_delay:
        return clients * one_client_txns_per_s
    else:
        return max_txns_per_s


if __name__ == "__main__":
    data = open(sys.argv[1])
    reader = csv.reader(data)
    table = []
    for row in reader:
        table.append(row)
    data.close()

    average = [["Simultaneous Clients", "Measured", '-95% confidence', '+95% confidence']]
    scatter = [('Simultaneous Clients', 'Transactions/s')]
    for row in table:
        x = int(row[0])
        for y in row[1:]:
            scatter.append((x, y))

        stats = stupidplot.stats([float(s) for s in row[1:]])
        average.append([x, stats[0], stats[0] - stats[-1], stats[0] + stats[-1]])


    # Add the model
    max_txns_per_s = max([row[1] for row in average[1:]])
    single_txns_per_s = average[1][1]
    average[0].append("Model")
    for row in average[1:]:
        row.append(model(row[0], single_txns_per_s, max_txns_per_s))

    options = {
        'plottype': 'points',
        'key': False,
        'ylabel': scatter[0][1],
        'yformat': '%g',
        'xformat': '%g',
        'xrange' : "[0:]",
    }
    stupidplot.gnuplotTable(scatter, "clients-scatter.eps", options)

    options = {
        #~ 'plottype': 'points',
        'key': "bottom right",
        'ylabel': scatter[0][1],
        'errorbars': [1],
        'yformat': '%g',
        'xformat': '%g',
        'xrange' : "[0:]",
    }
    stupidplot.gnuplotTable(average, "clients-average.eps", options)

    for row in average[1:]:
        print row[0], row[1], "+/-", float(row[1]) - float(row[2])
