#!/usr/bin/python

import csv
import sys

import stupidplot


def model_blocking(percentage, sp_txns_per_second, mp_txns_per_second):
    sp_seconds_per_transaction = 1/sp_txns_per_second
    mp_seconds_per_transaction = 1/mp_txns_per_second
    fraction = percentage/100.0
    combined_seconds_per_transaction = (fraction * mp_seconds_per_transaction +
            (1-fraction) * sp_seconds_per_transaction)
    return 1/combined_seconds_per_transaction


def model_speculative(percentage, sp_txns_per_second, mp_txns_per_second):
    # This model does the math *per partition*
    # 2 partitions: divide sp txns/s by 2
    sp_seconds_per_transaction = 1/(sp_txns_per_second / 2)
    # MP transaction involves both partitions equally
    mp_seconds_per_transaction = 1/mp_txns_per_second

    # Easier for me to think about computing the time for 100 transactions
    mp_count = percentage
    sp_count = (100 - mp_count) / 2.  # only execute half on each partition
     
    # Each mp transaction requires at least sp_seconds/2 of CPU on one partition
    # (an mp transaction reads/writes half the variables of an SP transaction on each partition)
    mp_processing_time = mp_count * (sp_seconds_per_transaction / 2.)
    sp_time = sp_count * sp_seconds_per_transaction

    # Each mp transaction introduces a stall that can be overlapped with sp transactions
    stall_time = mp_count * (mp_seconds_per_transaction - sp_seconds_per_transaction / 2.)
    # Assume we can perfectly overlap stall and sp execution
    sp_time -= stall_time
    if sp_time < 0:
        sp_time = 0
    assert sp_time >= 0

    return 100./(mp_processing_time + stall_time + sp_time)


def extractTables(path, title):
    data = open(path)
    reader = csv.reader(data)
    table = []
    for row in reader:
        table.append(row)
    data.close()

    scatter = [('% Multi-partition', 'Transactions/s')]
    average = [['% Multi-partition', title, '-95% confidence', '+95% confidence']]
    for row in table:
        x = int(row[0])
        for y in row[1:]:
            scatter.append((x, y))
        stats = stupidplot.stats([float(f) for f in row[1:]])
        average.append([x, stats[0], stats[0]-stats[-1], stats[0]+stats[-1]])

    return scatter, average


if __name__ == "__main__":
    scatter_block, average_block = extractTables(sys.argv[1], "Blocking")
    scatter_speculative, average_speculative = extractTables(sys.argv[2], "Speculative")


    # Add the model
    assert average_block[1][0] == 0
    sp_txns_per_second = average_block[1][1]
    mp_txns_per_second = average_block[-1][1]
    assert average_block[-1][0] == 100
    average_block[0].extend(['Blocking Model', 'Speculative Model'])
    for row in average_block[1:]:
        block = model_blocking(row[0], sp_txns_per_second, mp_txns_per_second)
        speculate = model_speculative(row[0], sp_txns_per_second, mp_txns_per_second)
        row.extend([block, speculate])
        print "\t".join([str(v) for v in row])

    options = {
        'plottype': 'points',
        'key': False,
        'ylabel': scatter_block[0][1],
        'yformat': '%g',
        'xformat': '%g',
    }
    stupidplot.gnuplotTable(scatter_block, "mptxns-scatter.eps", options)

    options = {
        #~ 'plottype': 'points',
        #~ 'key': False,
        'ylabel': 'Transactions/s',
        'errorbars': [1],
        'yformat': '%g',
        'xformat': '%g',
        #~ 'xrange' : "[0:]",
    }
    stupidplot.gnuplotTable([average_block, average_speculative], "mptxns-average.eps", options)

    #~ for row in average:
        #~ print "\t".join([str(s) for s in row])
