#!/bin/sh

set -e

UNNEEDED="""
2pc
mysqlengine
ping
protodtxn
protorpc
third_party/libdrizzle
third_party/protobuf
third_party/simplejson
"""

LINK="""
abort_plot.py
conflict.py
conflict_plot.py
mptxn.py
mptxn_plot.py
repeatability/build.sh
repeatability/doall.sh
tpcc_mptxn.py
tpcc_mptxn_plot.py
tpcc_warehouses.py
tpcc_warehouses_plot.py
"""

HSTORE_PATCHES="""
distributor-disable-timeout.diff
tpcc-no-bind-district.diff
linux-build.diff
"""

LOCK_PATCHES="""
timeout-800ms.diff
tpcc-no-bind-district.diff
linux-build-lock.diff
"""

# Check out all the repositories
hg clone ~/hstore/hstore-base hstore
hg clone hstore hstore-lock
hg clone ~/stupidbuild stupidbuild
hg clone ~/taskmgr taskmgr
hg clone ~/hstore/taskmgrconfigs taskmgrconfigs
mkdir data

# Patch the source and remove unneeded directories
cd hstore
for i in $HSTORE_PATCHES; do
    hg qim ../hstore/perfscripts/repeatability/$i
    hg qpush
done
rm -rf $UNNEEDED
cd ..

cd hstore-lock
hg up -C -r replication-after-execution
for i in $LOCK_PATCHES; do
    hg qim ../hstore/perfscripts/repeatability/$i
    hg qpush
done
rm -rf $UNNEEDED
cd ..

# Link all the useful scripts
for i in $LINK; do
    ln -s "hstore/perfscripts/$i" .
done

# Remove all mercurial state
rm -rf `ls -d */.hg */.hgignore */.hgtags`

# Output the documentation
~/6.092/tools/asciidoc/asciidoc.py --unsafe -a stylesdir=~/6.092 -a theme=plain -o README.html hstore/perfscripts/repeatability/README

cp ~/hstore-documentation/hstore-cc/hstore-cc-final.pdf .
