#!/usr/bin/python

import csv
import os.path
import subprocess
import time

import start_multicore

CLIENT = "build/htableclientperf"
DISTRIBUTOR_PORT = 12345
PERCENT_MULTIPARTITION = [0, 5, 10, 15, 20, 100]
TRIALS = 3

def runTest(host, base_port, max_partitions):
    partitions = 1
    table = []
    while partitions <= max_partitions:
        config = start_multicore.makeConfiguration(host, base_port, partitions)
        distributor = start_multicore.startDistributor(DISTRIBUTOR_PORT, config.name)
        time.sleep(0.25)  # Hack: wait for the distributor to start
        try:
            for percent_multipartition in PERCENT_MULTIPARTITION:
                row = [partitions, percent_multipartition]
                print "%d partitions %d%% MP:" % (partitions, percent_multipartition),
                sys.stdout.flush()

                distributor_address = "localhost:%d" % (DISTRIBUTOR_PORT)
                threads = 30
                dispatch = threads
                for _ in xrange(TRIALS):
                    args = (CLIENT, distributor_address, config.name, str(dispatch), str(threads), str(percent_multipartition))
                    client = subprocess.Popen(args, stdout=subprocess.PIPE)
                    output = client.stdout.read()
                    result = client.wait()
                    assert result == 0

                    transactions_per_second = float(output)
                    row.append(transactions_per_second)
                    print transactions_per_second,
                    sys.stdout.flush()

                table.append(row)
                print
        finally:
            start_multicore.stopServer(distributor)

        partitions *= 2

    return table


if __name__ == "__main__":
    import sys
    if len(sys.argv) != 5:
        sys.stderr.write("run_multicore.py host base_port max_partitions output\n")
        sys.exit(1)

    host = sys.argv[1]
    base_port = int(sys.argv[2])
    max_partitions = int(sys.argv[3])
    output_path = sys.argv[4]

    if os.path.exists(output_path):
        sys.stderr.write("%s exists: refusing to overwrite\n" % (output_path));
        sys.exit(1)

    table = runTest(host, base_port, max_partitions)
    out = open(output_path, "wb")
    writer = csv.writer(out)
    writer.writerows(table)
    out.close()

