#!/usr/bin/python

"""Tests HTable performance while scaling the % of multi-partition transactions."""

import csv
import os.path
import sys

from third_party.pystdlib import subprocess

PERF = "./htableclientperf"
PERF_LOCK = "./htableclientperf-lock"

TRIALS = 3
MAX_PARTITIONS = 2


def clientPerf(mode, distributor, configuration_file, dispatch, threads, percent_multipartition,
        conflicts=0, aborts=0):
    if mode == "lock":
        command = [PERF_LOCK, configuration_file, str(dispatch)]
    else:
        command = [PERF, distributor, configuration_file, str(dispatch), str(threads)]
    command.extend([str(percent_multipartition), str(conflicts)])
    if aborts != 0:
        assert conflicts == 0
        command.append(str(aborts))
    child = subprocess.Popen(command, stdout=subprocess.PIPE)
    output = child.stdout.read()
    result = child.wait()
    assert result == 0

    return float(output.strip())


def saveCSVResults(configuration_file, results):
    output_path = os.path.splitext(configuration_file)[0] + ".csv"
    out = open(output_path, "wb")
    writer = csv.writer(out)
    writer.writerows(results)
    out.close()


def runScaleMptxn(mode, configuration_file, distributor, conflicts=0, aborts=0):
    results = []

    partitions = MAX_PARTITIONS
    for percent_multipartition in range(0, 20, 2) + range(20, 101, 5):
        # 20 transactions per partition
        dispatch = partitions * 20
        threads = dispatch

        row = [percent_multipartition]
        print "%d%% multipartition" % row[0], 
        sys.stdout.flush()
        for _ in range(TRIALS):
            output = clientPerf(mode, distributor, configuration_file, dispatch, threads,
                    percent_multipartition, conflicts=conflicts, aborts=aborts)
            row.append(output)
            print "\t", row[-1],
            sys.stdout.flush()
        results.append(row)
        print

    return results


if __name__ == "__main__":
    if len(sys.argv) != 4:
        sys.stderr.write("scale_mptxns.py [mode] [config file] [distributor]\n")
        sys.exit(1)

    mode = sys.argv[1]
    configuration_file = sys.argv[2]
    distributor = sys.argv[3]

    results = runScaleMptxn(mode, configuration_file, distributor)
    saveCSVResults(configuration_file, results)
