#!/usr/bin/python

"""Perform the % conflict scaling experiment."""

import csv
import os.path
import sys

import scale_mptxns
from third_party.pystdlib import subprocess

def runTest(test_mode, server_mode, distributor, configuration_file):
    assert test_mode == "conflict" or test_mode == "abort"
    results = []

    if test_mode == "conflict":
        percent_range = xrange(0, 101, 20)
    else:
        percent_range = (0, 1, 3, 5, 10)
    # Commented range is for aborts
    for percent in percent_range:
        print "%d%% %s" % (percent, test_mode)
        if test_mode == "conflict":
            kwargs = { "conflicts": percent }
        else:
            kwargs = { "conflicts": 0, "aborts": percent }
        scale_results = scale_mptxns.runScaleMptxn(
                server_mode, configuration_file, distributor,
                **kwargs)

        for row in scale_results:
            out_row = [percent]
            out_row.extend(row)
            results.append(out_row)
        print

    output_path = os.path.splitext(configuration_file)[0] + ".csv"
    out = open(output_path, "wb")
    writer = csv.writer(out)
    writer.writerows(results)
    out.close()

if __name__ == "__main__":
    if len(sys.argv) != 5:
        sys.stderr.write("scale_mptxns_conflict.py [conflict|abort] [block|spec|lock] [distributor] [config file]\n")
        sys.exit(1)

    test_mode = sys.argv[1]
    server_mode = sys.argv[2]
    distributor = sys.argv[3]
    configuration_file = sys.argv[4]

    runTest(test_mode, server_mode, distributor, configuration_file)
