#!/usr/bin/python

import csv
import subprocess

WAREHOUSES_PER_CPU = 2
MAX_CPUS = 64
TRIALS = 3

COMMAND = "./tpcc-scale"


def executeTpccScale(warehouses_per_cpu, cpus):
    """Returns the TPC-C transactions/second."""

    proc = subprocess.Popen(
            (COMMAND, str(warehouses_per_cpu), str(cpus)), stdout=subprocess.PIPE)
    output = proc.stdout.read()
    result = proc.wait()
    assert result == 0

    return float(output.split()[0])


def main():
    table = []
    for cpus in xrange(1, MAX_CPUS+1):
        print cpus,
        row = [cpus]
        for _ in xrange(TRIALS):
            qps = executeTpccScale(WAREHOUSES_PER_CPU, cpus)
            print qps,
            row.append(qps)
        print
        table.append(row)

    out = open("scale_tpcc_cpus.csv", "wb")
    writer = csv.writer(out)
    writer.writerows(table)
    out.close()


def constant_datasize():
    table = []
    # Can't run 64 warehouses in a 32-bit process: not enough RAM
    WAREHOUSES = 32
    cpus = 1
    while cpus <= MAX_CPUS and cpus <= WAREHOUSES:
        warehouses_per_cpu = WAREHOUSES / int(cpus)
        assert warehouses_per_cpu * cpus == WAREHOUSES

        print "%d cpus %d warehouses/cpu" % (cpus, warehouses_per_cpu),
        row = [cpus]

        for _ in xrange(TRIALS):
            qps = executeTpccScale(warehouses_per_cpu, cpus)
            print qps,
            row.append(qps)
        print
        table.append(row)

        # Double the number of CPUs
        cpus <<= 1

    out = open("scale_tpcc_cpus_constant.csv", "wb")
    writer = csv.writer(out)
    writer.writerows(table)
    out.close()


if __name__ == "__main__":
    main()
    constant_datasize()
