#!/usr/bin/python

import errno
import os
import signal
import subprocess
import tempfile


SERVER = "build/htableserver"
DISTRIBUTOR = "build/htabledistributor"

def makeConfiguration(ip, base_port, num_processes):
    out = tempfile.NamedTemporaryFile()
    for i in xrange(num_processes):
        out.write("foo\n%s:%d\n\n" % (ip, base_port + i))
    out.flush()
    return out


def startServer(cpu, port, mode ):
    # Start the server, wait for it to start listening
    args = ("taskset", "--cpu-list", str(cpu), SERVER, str(port), mode)
    #~ print args
    return subprocess.Popen(args, stdout=subprocess.PIPE)


def startDistributor(port, configuration):
    # Start the server, wait for it to start listening
    args = (DISTRIBUTOR, str(port), configuration)
    #~ print args
    return subprocess.Popen(args, stdout=subprocess.PIPE)


def stopServer(server):
    if server.poll() is None:
        try:
            os.kill(server.pid, signal.SIGTERM)
        except OSError, e:
            if e.errno != errno.ESRCH:
                raise
        server.wait()


def runServers(base_port, mode, processes):
    servers = []
    try:
        for i in xrange(processes):
            port = base_port + i
            servers.append(startServer(i, port, mode))

        #~ config = makeConfiguration("localhost", base_port, processes)
        #~ servers.append(startDistributor(base_port + processes, config.name))
        #~ print "distributor listening on %d" % (base_port + processes)
        
        pid, status = os.wait()
        print "process %d exited status %d" % (pid, status)

        # Remove the dead process from our list
        found = False
        for i, server in enumerate(servers):
            if server.pid == pid:
                found = True
                del servers[i]
                break
        assert found

    finally:
        print "stopping servers ..."
        for server in servers:
            stopServer(server)

if __name__ == "__main__":
    import sys
    if len(sys.argv) != 4:
        sys.stderr.write("start_multicore.py base_port mode processes\n")
        sys.exit(1)
    
    base_port = int(sys.argv[1])
    mode = sys.argv[2]
    processes = int(sys.argv[3])

    runServers(base_port, mode, processes)
