// Copyright 2008,2009,2010 Massachusetts Institute of Technology.
// All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

#include <cstdio>
#include <cstdlib>

#include "base/cast.h"
#include "dtxn/common.h"
#include "dtxn/configparser.h"
#include "tpcccoordinator.h"

int main(int argc, const char* argv[]) {
    if (argc != 4) {
        fprintf(stderr, "tpccdistributor [listen port] [configuration file] [num warehouses]\n");
        return 1;
    }
    int port = atoi(argv[1]);
    assert(port != 0);
    const char* const config_file = argv[2];
    int num_warehouses = atoi(argv[3]);
    assert(num_warehouses > 0);

    // Parse the config file to get the number of partitions
    int num_partitions = assert_range_cast<int>(
            dtxn::parseConfigurationFromPath(config_file).size());
    TPCCCoordinator distributor(num_warehouses, num_partitions);
    dtxn::runDistributor(&distributor, port, config_file);

    return 0;
}
