#!/usr/bin/python

import subprocess
import sys

# TODO: Read these from taskmgrconfigs/task_mptxn.py
REPLICAS = 2
PARTITIONS = 2


class Slaves(object):
    def __init__(self, script):
        num_servers = REPLICAS * PARTITIONS + 2
        if len(sys.argv) < num_servers + 1:
            sys.stderr.write("%s [%d hosts to use to run the test]\n" % (sys.argv[0], num_servers))
            sys.stderr.write("NOTE: Must have passwordless ssh access to each host\n")
            sys.exit(1)

        self.hosts = tuple(sys.argv[1:num_servers+1])
        self.script = script
        self._extra_args = sys.argv[num_servers+1:]

    def start(self):
        print "starting taskmgr slaves ..."
        run(("taskmgr/slave_admin.py", "start") + self.hosts)

    def taskcmd(self, args):
        run(taskcmd(self.script, self.hosts, args))

    def wait(self, args):
        return runCapture(
                taskcmd(self.script, self.hosts, ("wait", self.hosts[-1], "--task_args") + args))

    def copyResults(self, destination):
        run(("scp", self.hosts[-1] + ":/tmp/tmp*/*.csv", destination))

    def stop(self):
        print "stopping taskmgr slaves ..."
        run(("taskmgr/slave_admin.py", "stop") + self.hosts)

    def extra_args(self):
        return self._extra_args


def run(command):
    print " ".join(command)
    cmd = subprocess.Popen(command)
    code = cmd.wait()
    assert code == 0


def runCapture(command):
    print " ".join(command)
    cmd = subprocess.Popen(command, stdout=subprocess.PIPE)
    output = cmd.stdout.read()
    code = cmd.wait()
    assert code == 0
    return output


def taskcmd(script, hosts, args):
    return ("taskmgr/taskcmd.py", "--slaves=" + ",".join(hosts), script) + tuple(args)


if __name__ == "__main__":
    slaves = Slaves("taskmgrconfigs/task_mptxn.py")
    slaves.start()

    extra_args = ()
    if len(slaves.extra_args()) == 1:
        assert slaves.extra_args()[0] == "general"
        extra_args = (slaves.extra_args()[0],)
    else:
        assert len(slaves.extra_args()) == 0

    # Run each test
    for test in ("block", "spec", "lock"):
        # Start the jobs
        print "starting %s test ..." % (test)
        slaves.taskcmd(("start", "--task_args", test) + extra_args)

        # Wait for the test to finish
        print "waiting for test to finish ..."
        slaves.wait((test,))

        print "copying results ..."
        test_csv_name = "data/mptxn-" + test
        if len(extra_args) == 1:
            # Name the "general transaction" output
            test_csv_name += "-general"
        slaves.copyResults(test_csv_name + ".csv");

        print "stoping %s test ..." % (test)
        slaves.taskcmd(("stop", "--task_args", test) + extra_args)

    slaves.stop()
