#!/usr/bin/python

import os
import sys

import dataextract
import stupidplot

INPUTS = [
    ["data/mptxn-spec.csv", "Speculation"],
    ["data/mptxn-block.csv", "Blocking"],
    ["data/mptxn-lock.csv", "Locking"],
]

#~ OCC = ["data/occ-mptxns-replicated.csv", "Optimistic"]

X_AXIS = "Multi-Partition Transactions"
Y_AXIS = "Transactions/second"

BLACK = 'rgb "black"'
RED = 'rgb "#c02626"'
GREEN = 'rgb "#14ba14"'
BLUE = 'rgb "#4242d8"'
MAGENTA = 'rgb "#c02adc"'

DEFAULT_PLOT_OPTIONS = {
    'dashed': True,
    'color': True,
    'colors': {
            1: BLACK,
            2: RED,
            3: GREEN,
            4: BLUE,
            5: MAGENTA,
    },
    #~ 'key': False,
    'ylabel': Y_AXIS,
    #~ 'errorbars': [1],
    'yformat': '%g',
    'xformat': '%g%%',
    'yrange' : '[0:]',
    'linewidth': 3,
    'grid': 'noxtics noytics',
    'fontsize': 22,
}


def extractTables(path, x_axis_title, y_axis_title, line_title):
    """Returns two tables containing the values from the CSV file in path. The first contains the
    the columns (x, average, -95% confidence, +95% confidence). The second contains (x, y)
    scatter plot values."""

    table = dataextract.readCSVTable(path)

    scatter = [(x_axis_title, y_axis_title)]
    average = [(x_axis_title, line_title, "-95% confidence", "+95% confidence")]
    for row in table:
        x = float(row[0])
        for y in row[1:]:
            scatter.append((x, y))
        stats = stupidplot.stats([float(f) for f in row[1:]])
        average.append([x, stats[0], stats[0]-stats[-1], stats[0]+stats[-1]])

    return average, scatter


def removeErrorBars(intable):
    out_table = []
    for i, row in enumerate(intable):
        out_table.append(row[:2])
    return out_table


def get_output():
    """Parses the output path from the command line, plus error validation."""

    if len(sys.argv) != 2:
        sys.stderr.write("%s output.eps\n" % (sys.argv[0]))
        sys.exit(1)

    output = sys.argv[1]
    if os.path.exists(output):
        sys.stderr.write("%s exists, refusing to overwrite (move it/delete it)\n" % output)
        sys.exit(1)

    return output


def plot(inputs, output_path):
    averages = []
    for path, line_title in inputs:
        average, scatter = extractTables(path, X_AXIS, Y_AXIS, line_title)
        averages.append(removeErrorBars(average))
    options = dict(DEFAULT_PLOT_OPTIONS)
    #~ options["key"] = False
    stupidplot.gnuplotTable(averages, output_path, options)


if __name__ == "__main__":
    if len(sys.argv) == 3:
        assert sys.argv[2] == "general"
        del sys.argv[2]
        for input_row in INPUTS:
            input_row[0] = input_row[0][:-4] + "-general.csv"
            input_row[1] = input_row[1] + " General"

    output_path = get_output()
    plot(INPUTS, output_path)
    #~ print "wtf", INPUTS + [OCC]
    #~ plot(INPUTS + [OCC], output_path[:-4] + "-occ.eps")
