#!/usr/bin/python

import os.path

import buildparser
import stupidbuild


if __name__ == "__main__":
    # Get the current dir then change to root to avoid bugs with paths
    currentDirectory = os.getcwd()
    os.chdir("/")
    
    # Parse the rules for the current directory
    parser = buildparser.BuildParser()
    parser.loadRules(currentDirectory)

    # Feed the rules into the build graph
    builder = stupidbuild.StupidBuilder(parser.root)
    for rule in parser.rules:
        builder.buildTarget(*rule)

    def trimPath(path):
        assert path[0] == '/'
        return path[1:]

    allOutputs = " ".join(["$(outdir)/" + trimPath(output) for output in builder.outputs])

    allDirectories = {}
    for output in builder.outputs:
        directory = os.path.dirname(trimPath(output))
        allDirectories[directory] = 1

    mkDirectories = ""
    for directory in allDirectories:
        mkDirectories += "\tmkdir -p $(outdir)/" + directory + "\n"

    print """outdir=build

CCFLAGS=-g -DHAVE_CONFIG_H -Werror -Wall -Wextra -Wshadow -Wpointer-arith -Wcast-align -Wwrite-strings -Wno-sign-compare -Wno-unused-parameter

all: $(outdir) %s

$(outdir):
%s

clean:
\trm -rf $(outdir)

CC_COMPILE=$(CC) $(CCFLAGS) -c -o $@ $< -I.
CC_LIBRARY=ar rcs $@ $^
CC_BINARY=g++ -o $@ $^ -lrt

""" % (allOutputs, mkDirectories)

    for output, node in builder.outputs.iteritems():

        trimmedInputs = []
        for input in node.inputs:
            if input in builder.outputs:
                trimmedInputs.append("$(outdir)/" + trimPath(input))
            else:
                trimmedInputs.append(trimPath(input))

        print "$(outdir)/%s: %s" % (trimPath(output), " ".join(trimmedInputs))
        _, extension = os.path.splitext(output)
        if extension == ".o":
            print "\t$(CC_COMPILE)"
        elif extension == ".a":
            print "\t$(CC_LIBRARY)"
        elif extension == "":
            print "\t$(CC_BINARY)"
        else:
            raise "Unknown extension: " + extension

