#!/usr/bin/python

import os
import subprocess
import sys


DESTINATION_DIR = "/tmp"

SCRIPT_DIR = os.path.dirname(__file__)
TASKMGRTAR_BASE = "taskmgr_local.tar.bz2"
TASKMGRTAR = os.path.join(SCRIPT_DIR, TASKMGRTAR_BASE)
PORT = 55555

def start(host):
    # Copy the file
    scp = subprocess.Popen(("scp", "-q", TASKMGRTAR, host + ":" + DESTINATION_DIR))
    code = scp.wait()
    assert code == 0

    # Start the command
    ssh = subprocess.Popen(("ssh", host, """
        killall taskmgr_slave.py &> /dev/null;
        cd /tmp;
        tar xjf %s;
        rm %s;
        taskmgr_slave/taskmgr_slave.py %d 0<&- &> taskmgr_slave/taskmgr_slave.log &""" % (
        TASKMGRTAR_BASE, TASKMGRTAR_BASE, PORT)))
    code = ssh.wait()
    assert code == 0


def stop(host):
    # Start the command
    ssh = subprocess.Popen(("ssh", host, "killall taskmgr_slave.py; rm -rf %s/taskmgr" % (
            DESTINATION_DIR)))
    code = ssh.wait()
    assert code == 0


def printHelp():
    print "slave_admin.py [start|stop] [host list]"
    print "  NOTE: You must have passwordless ssh access to each host and write access"
    print "        to %s" % DESTINATION_DIR


if __name__ == "__main__":
    if not os.path.exists(TASKMGRTAR):
        sys.stderr.write("Error: %s does not exist\n" % TASKMGRTAR)
        sys.stderr.write("(maybe you need to run ./makelocal.sh?)\n")
        sys.exit(1)

    if len(sys.argv) < 3:
        printHelp()
        sys.exit(1)

    mode = sys.argv[1]
    hostlist = sys.argv[2:]

    if mode == "start":
        for host in hostlist:
            start(host)
    elif mode == "stop":
        for host in hostlist:
            stop(host)
    else:
        sys.stderr.write('Error: mode must be one of "start" or "stop"\n\n')
        printHelp()
        sys.exit(1)

    sys.exit(0)
