#!/usr/bin/python

import errno
import os
import time
import unittest

import slave
import task_test


class SlaveTest(unittest.TestCase):
    def testStartStatusStop(self):
        s = slave.Slave()
        task_id = s.taskStart(("ls",), task_test.EMPTY_TAR)

        # Poll until task has exited
        while s.taskState(task_id) == None:
            pass

        self.assertEquals(0, s.taskState(task_id))
        self.assertEquals(0, s.taskStop(task_id))
        self.assertRaises(KeyError, s.taskState, task_id)

    def testStartPackage(self):
        package_data = task_test.makeTar("one", "ok")
        s = slave.Slave()
        task_id = s.taskStart(("ls",), package_data)

        # Poll until task has exited
        while s.taskState(task_id) == None:
            pass

        self.assertEquals(0, s.taskState(task_id))
        self.assertEquals("one\n", s.taskOutput(task_id))
        self.assertEquals(0, s.taskStop(task_id))

    def testShutdown(self):
        s = slave.Slave()
        task_id = s.taskStart(("cat",), task_test.EMPTY_TAR)

        # Wait for the task to actually start
        # TODO: We should do this in a clean fashion using pipes with close-on-exec?
        time.sleep(0.01)

        pid = s.tasks[task_id].child_pid
        directory = s.tasks[task_id].package.path()
        # Kill with signal 0: perform error checking
        self.assertEquals(None, os.kill(pid, 0))
        self.assertTrue(os.path.exists(directory))

        s.shutdown()
        try:
            os.kill(pid, 0)
            self.fail("Should have raised ESRCH")
        except OSError, e:
            self.assertEquals(errno.ESRCH, e.errno)
        self.assertFalse(os.path.exists(directory))

    def testList(self):
        s = slave.Slave()
        t1 = s.taskStart(("ls",), task_test.EMPTY_TAR)
        t2 = s.taskStart(("ls",), task_test.EMPTY_TAR)
        s.tasks[t1].wait()
        s.tasks[t2].wait()

        value = [
                (t1, ("ls",), ("EXITED", "Exit status code: 0")),
                (t2, ("ls",), ("EXITED", "Exit status code: 0")),
        ]
        self.assertEquals(value, s.taskList())

    def testWait(self):
        s = slave.Slave()
        t1 = s.taskStart(("echo", "wtf"), task_test.EMPTY_TAR)
        code, output = s.taskWait(t1)
        self.assertEquals(0, code)
        self.assertEquals("wtf\n", output)


if __name__ == "__main__":
    unittest.main()
