#!/usr/bin/python

import errno
import signal
import SimpleXMLRPCServer
import select
import sys

import taskmgr.slave

__pychecker__ = "unusednames=stack_trace"

class SlaveWrapper(taskmgr.slave.Slave):
    def taskStart(self, command, package_data):
        return taskmgr.slave.Slave.taskStart(self, command, package_data.data)


class StoppableXMLRPCServer(SimpleXMLRPCServer.SimpleXMLRPCServer):
    """Server subclass that can be stopped."""

    def __init__(self, listen_address):
        SimpleXMLRPCServer.SimpleXMLRPCServer.__init__(self, listen_address)
        self._serve = True

    def serve_forever(self):
        """Serve until told to stop."""
        self._serve = True
        while self._serve:
            try:
                self.handle_request()
            except select.error, e:
                # Ignore EINTR: caused by catching SIGTERM (I hope)
                if e.args[0] != errno.EINTR:
                    raise
                else:
                    assert not self._serve

    def stop_serving(self):
        """Stop serving."""

        assert self._serve is True
        self._serve = False
        self.server_close()


def main():
    port = int(sys.argv[1])

    slave = SlaveWrapper()

    xmlrpc_server = StoppableXMLRPCServer(('', port))
    xmlrpc_server.logRequests = False
    xmlrpc_server.register_instance(slave)

    # Catch SIGTERM and exit cleanly
    def exitOnSigterm(signum, stack_trace):
        print "SIGTERM: Exiting"
        assert signum == signal.SIGTERM
        xmlrpc_server.stop_serving()
    signal.signal(signal.SIGTERM, exitOnSigterm)

    xmlrpc_server.serve_forever()
    slave.shutdown()


if __name__ == "__main__":
    main()
