#! /bin/sh
#
# init.d script for taskmgr_slave.py. Works on Planetlab.
#
# chkconfig: 2345 80 15
# description: Starts taskmgr_slave for distributed task management.
### BEGIN INIT INFO
# Provides:          taskmgr_slave
# Required-Start:    
# Required-Stop:     
# Default-Start:     2 3 4 5
# Default-Stop:      0 1 6
# Short-Description: taskmgr_slave for distributed task management.
# Description:       taskmgr_slave for distributed task management.
### END INIT INFO

# Author: Evan Jones <evanj@mit.edu>


PORT=55555

NAME=taskmgr_slave


# PATH should only include /usr/* if it runs after the mountnfs.sh script
PATH=/sbin:/usr/sbin:/bin:/usr/bin
DESC="distributed task manager"
DAEMON_ARGS="$PORT"
SCRIPTNAME=/etc/init.d/$NAME

#~ LOG=$NAME.log
#~ PIDFILE=$NAME.pid
#~ DAEMON=./$NAME.py
LOG=/var/log/$NAME.log
PIDFILE=/var/run/$NAME.pid
DAEMON=/usr/local/bin/$NAME.py


#
# Function that starts the daemon/service
#
do_start()
{
    if [ -e $PIDFILE ]; then
        # Either the server is already running, or we rebooted
        PID=`cat $PIDFILE`
        if ! kill -0 $PID &> /dev/null; then
            # Process does not exist: continue
            rm $PIDFILE
        else
            echo "ERROR: PID file $PIDFILE exists; process PID $PID is running" 1>&2
            exit 1
        fi
    fi
    # Close STDIN, redirect STDOUT and STDERR to $LOG
    sudo -u nobody $DAEMON $DAEMON_ARGS 0<&- &> $LOG &
    PID=$!
    echo $PID > $PIDFILE
}


#
# Function that stops the daemon/service
#
do_stop()
{
    if [ ! -r $PIDFILE ]; then
        echo "ERROR: Cannot read $PIDFILE" 1>&2
        exit 1
    fi

    PID=`cat $PIDFILE`
    kill $PID
    rm $PIDFILE
    return 0
}

if [ $UID != 0 ]; then
    echo "ERROR: Must be root" 1>&2
    exit 1
fi

case "$1" in
  start)
    echo "Starting $DESC" "$NAME"
    do_start
    ;;
  stop)
    echo "Stopping $DESC" "$NAME"
    do_stop
    ;;
  restart|force-reload)
    #
    # If the "reload" option is implemented then remove the
    # 'force-reload' alias
    #
    echo "Restarting $DESC" "$NAME"
    do_stop
    do_start
    ;;
  *)
    #echo "Usage: $SCRIPTNAME {start|stop|restart|reload|force-reload}" >&2
    echo "Usage: $SCRIPTNAME {start|stop|restart|force-reload}" >&2
    exit 3
    ;;
esac

:
