#!/usr/bin/python

import os.path
import tempfile

import taskmgr.task


PARTITIONS = 1
REPLICAS = 2
PORT = 12345

HOST_LIST = []
for i in xrange(5, 14):
    HOST_LIST.append("farm%d.csail.mit.edu:55555" % (i))


def makeConfiguration(partition_lists):
    f = tempfile.NamedTemporaryFile()
    for partition in partition_lists:
        f.write("partition function\n")
        for host in partition:
            f.write("%s:%d\n" % (host, PORT))
        f.write("\n")
    f.flush()
    return f


def htableserverCommand(config_tempfile, partition_index, replica_index):
    config_path = os.path.basename(config_tempfile.name)
    return ("taskset", "--cpu-list", "0", "./htableserver", "block", config_path,
            str(partition_index), str(replica_index))


def scheduleTasks(scheduler):
    # Reserve all the hosts for the servers
    partitions = []
    for partition_index in xrange(PARTITIONS):
        partition_hosts = []
        for replica_index in xrange(REPLICAS):
            partition_hosts.append(scheduler.reserveHost())
        partitions.append(partition_hosts)

    # Write the config file
    config = makeConfiguration(partitions)
    server_package = taskmgr.task.makeTarFromFiles(["htable-opt/htableserver", config.name])

    # Schedule backups first, so they are running when we start the primaries
    for partition_index in xrange(PARTITIONS):
        for replica_index in xrange(1, REPLICAS):
            command = htableserverCommand(config, partition_index, replica_index)
            scheduler.scheduleHost(partitions[partition_index][replica_index], command, server_package)

    # Schedule primaries
    for partition_index in xrange(PARTITIONS):
        command = htableserverCommand(config, partition_index, 0)
        scheduler.scheduleHost(partitions[partition_index][0], command, server_package)

    #~ client_package = taskmgr.task.makeTarFromFiles([
        #~ "htable-opt/htableclientperf",
        #~ "../hstore/hstore-base/perfscripts/scale_mptxns.py",
        #~ "../hstore/hstore-base/perfscripts/scale_clients.py",
        #~ config.name], {
        #~ "../hstore/hstore-base/third_party/__init__.py": "third_party/__init__.py",
        #~ "../hstore/hstore-base/third_party/pystdlib": "third_party/pystdlib",
    #~ })
    #~ client_command = ("./scale_clients.py", os.path.basename(config.name), "%s:12345" % partitions[0][0])
    client_package = taskmgr.task.makeTarFromFiles([
        "htable-opt/htableclientperf",
        "htable-opt/perfloop.sh",
        config.name,
    ])
    client_command = ("./perfloop.sh", "%s:12345" % partitions[0][0], os.path.basename(config.name))
    client_host = scheduler.schedule(client_command, client_package)
