#!/usr/bin/python

import os.path
import tempfile

import taskmgr.task


PARTITIONS = 2
REPLICAS = 2
PORT = 12345

HOST_LIST = []
for i in xrange(5, 15):
    HOST_LIST.append("farm%d.csail.mit.edu:55555" % (i))


def makeConfiguration(partition_lists):
    f = tempfile.NamedTemporaryFile()
    for partition in partition_lists:
        f.write("partition function\n")
        for host in partition:
            f.write("%s:%d\n" % (host, PORT))
        f.write("\n")
    f.flush()
    return f


def htableserverCommand(config_tempfile, partition_index, replica_index):
    config_path = os.path.basename(config_tempfile.name)
    return ("taskset", "--cpu-list", "0", "./htableserver", "spec", config_path,
    #~ return ("taskset", "--cpu-list", "0", "./htableserver-lock", "lock", config_path,
            str(partition_index), str(replica_index))


def scheduleTasks(scheduler):
    # Reserve all the hosts for the servers
    partitions = []
    for partition_index in xrange(PARTITIONS):
        partition_hosts = []
        for replica_index in xrange(REPLICAS):
            partition_hosts.append(scheduler.reserveHost())
        partitions.append(partition_hosts)

    # Write the config file
    config = makeConfiguration(partitions)
    server_package = taskmgr.task.makeTarFromFiles(["build/htableserver", config.name])
    #~ server_package = taskmgr.task.makeTarFromFiles(["build/htableserver-lock", config.name])

    # Schedule backups first, so they are running when we start the primaries
    for partition_index in xrange(PARTITIONS):
        for replica_index in xrange(1, REPLICAS):
            command = htableserverCommand(config, partition_index, replica_index)
            scheduler.scheduleHost(partitions[partition_index][replica_index], command, server_package)

    # Schedule primaries
    for partition_index in xrange(PARTITIONS):
        command = htableserverCommand(config, partition_index, 0)
        scheduler.scheduleHost(partitions[partition_index][0], command, server_package)

    # Schedule distributor
    distributor_package = taskmgr.task.makeTarFromFiles([
        #~ "htable-rep-debug/htabledistributor-occ",
        "build/htabledistributor",
        config.name])
    #~ distributor_command = ("taskset", "--cpu-list", "0", "./htabledistributor-occ", str(PORT), os.path.basename(config.name))
    distributor_command = ("taskset", "--cpu-list", "0", "./htabledistributor", str(PORT), os.path.basename(config.name))
    distributor_host = scheduler.schedule(distributor_command, distributor_package)

    client_package = taskmgr.task.makeTarFromFiles([
        #~ "build/htableclientperf-lock",
        #~ "scale_mptxns.py",
        #~ "htable-rep-debug/htableclientperf-occ",
        "build/htableclientperf",
        "../hstore-base/perfscripts/scale_mptxns.py",
        "../hstore-base/perfscripts/scale_clients.py",
        #~ "../hstore/hstore-base/perfscripts/scale_mptxns_conflict.py",
        config.name], {
        "../hstore-base/third_party/__init__.py": "third_party/__init__.py",
        "../hstore-base/third_party/pystdlib": "third_party/pystdlib",
    })
    client_command = ("./scale_mptxns.py", os.path.basename(config.name), "%s:12345" % distributor_host)
    #~ client_command = ("./scale_clients.py", os.path.basename(config.name), "%s:12345" % partitions[0][0])
    #~ client_command = ("./scale_mptxns_conflict.py", os.path.basename(config.name))
    client_host = scheduler.schedule(client_command, client_package)
