#!/usr/bin/python

import os.path

import taskmgr.task

import task_mptxn


def scheduleClient(scheduler, client_command, config):
    client_package = taskmgr.task.makeTarFromFiles([
        task_mptxn.SOURCE_PATH + "/build/tpccclientperf",
        task_mptxn.LOCK_SOURCE_PATH + "/build/tpccclientperf-lock",
        task_mptxn.SOURCE_PATH + "/perfscripts/frange.py",
        task_mptxn.SOURCE_PATH + "/perfscripts/scale_tpcc_mptxns.py",
        config.name], {
        task_mptxn.SOURCE_PATH + "/third_party/__init__.py": "third_party/__init__.py",
        task_mptxn.SOURCE_PATH + "/third_party/pystdlib": "third_party/pystdlib",
    })
    client_host = scheduler.schedule(client_command, client_package)
    return client_host


def startTpcc(scheduler, args):
    assert len(args) == 2
    mode = args[0]
    warehouses = int(args[1])

    # Reserve all the hosts for the servers
    partitions = task_mptxn.makePartitions(scheduler)

    # Write the config file
    config = task_mptxn.makeConfiguration(partitions)

    distributor_host = task_mptxn.scheduleServersDistributors(
            scheduler, partitions, "tpccserver", mode, (str(warehouses),), "tpccdistributor",
            config)

    return mode, warehouses, config, distributor_host


def scheduleTasks(scheduler, args):
    mode, warehouses, config, distributor_host = startTpcc(scheduler, args)

    if mode == "lock":
        client_command = ("./tpccclientperf-lock",)
    else:
        client_command = ("./tpccclientperf", "%s:%d" % (distributor_host, task_mptxn.PORT))
    client_command += (os.path.basename(config.name), str(warehouses), str(20))
    scheduleClient(scheduler, client_command, config)
