#!/usr/bin/python

import csv
import sys
import time

import mptxn

# It takes 70 seconds to load 10 warehouses
DELAY_PER_WAREHOUSE = int(70/10)

MAX_WAREHOUSES = 20
TRIALS = 3


def serverIdsReplicasFirst(num_replicas, num_partitions):
    """Returns a list of server ids, with replicas first followed by primaries."""
    #Primaries are numbered 0, num_replicas, 
    out = []
    for partition in xrange(num_partitions):
        for i in xrange(1, num_replicas):
            out.append(partition * num_replicas + i)

    for partition in xrange(num_partitions):
        out.append(partition * num_replicas)

    return out


def distributorClientIds(num_replicas, num_partitions):
    servers = num_replicas * num_partitions
    return (servers, servers+1)


def start(slaves, server_index, mode, warehouses):
    slaves.taskcmd(("start", slaves.hosts[server_index], "--task_args", mode, str(warehouses)))


def tpccStart(slaves, mode, warehouses):
    # Wait for servers to load the data: start replicas then primaries
    for server_index in serverIdsReplicasFirst(mptxn.REPLICAS, mptxn.PARTITIONS):
        start(slaves, server_index, mode, warehouses)
    time.sleep(DELAY_PER_WAREHOUSE * warehouses / 2)

    # Start the distributor and the client
    for server_index in distributorClientIds(mptxn.REPLICAS, mptxn.PARTITIONS):
        start(slaves, server_index, mode, warehouses)
    

if __name__ == "__main__":
    slaves = mptxn.Slaves("taskmgrconfigs/task_tpcc_warehouses.py")
    slaves.start()

    # Run each test
    for mode in ("block", "spec", "lock"):
        results = []
        for warehouses in xrange(2, MAX_WAREHOUSES+1, 2):
            row = [warehouses]
            for trial in xrange(TRIALS):
                tpccStart(slaves, mode, warehouses)

                # Wait for the test to finish
                output = slaves.wait((mode, str(warehouses)))
                txns_per_second = float(output.strip())
                row.append(txns_per_second)
                print txns_per_second,
                sys.stdout.flush()

                slaves.taskcmd(("stop", "--task_args", mode, str(warehouses)))

            results.append(row)

        out = open("data/tpcc-warehouses-%s.csv" % (mode), "wb")
        writer = csv.writer(out)
        writer.writerows(results)
        out.close()

    # Stop the slaves
    slaves.stop()
